
local Healcsi = Healcsi
local Gui     = Healcsi.Gui
local L       = Healcsi.Locale

local function SlashProcessor(input)
	if (Healcsi.configWindow:IsShown()) then
		Healcsi.configWindow:Hide()
	else
		Healcsi.configWindow:Show()
	end
end

local function ConfigWindow_SizeChanged(self, width, height)
	-- nothing
end

local function ConfigWindow_Changed(self, event, value)
	if (event == "MenuChanged") then
		if (value == "profiles") then
			
			self.profileEditor.frame:Show()
			self.modulesEditor.frame:Hide()

			-- This is where I implemented the layout... dumb? yes! works? yes ;)
			self.profileEditor.frame:ClearAllPoints()
			self.profileEditor.frame:SetPoint("TOPLEFT", self.content)
			self.profileEditor.frame:SetPoint("BOTTOMRIGHT", self.content)


		elseif (value == "modules") then
			
			self.profileEditor.frame:Hide()
			self.modulesEditor.frame:Show()

			-- This is where I implemented the layout... dumb? yes! works? yes ;)
			self.modulesEditor.frame:ClearAllPoints()
			self.modulesEditor.frame:SetPoint("TOPLEFT", self.content)
			self.modulesEditor.frame:SetPoint("BOTTOMRIGHT", self.content)

		end
	elseif (event == "OnShow") then
		Healcsi:ModuleSelectorInit()
		self:UpdateMenu()
	end
end

local loaded = CreateFrame("Frame")
loaded:RegisterEvent("ADDON_LOADED")
loaded:SetScript("OnEvent",function(self, event, addon)
	Healcsi:BroadcastEvent("GlobalConfigRefresh") -- so we can can update stuff like macros
end)

local function OnProfileCallback(event, db, ...)
	--Healcsi:BroadcastEvent("ProfileChanged")
	Healcsi:BroadcastEvent("GlobalConfigRefresh")
end

local mediaResourceFrame = CreateFrame("Frame")
mediaResourceFrame.lifespan = 60 -- keep is alive for 1 minute!

mediaResourceFrame:SetScript("OnUpdate", function(self, elapsed)
	self.timer = (self.timer or 0) + elapsed
	if (self.timer > 1 and self.newMediaRegistered) then
		self.timer = 0
		self.newMediaRegistered = nil
		Healcsi:BroadcastEvent("GlobalConfigRefresh") -- so we don't miss media goodies
	end
	self.lifespan = self.lifespan - elapsed
	if (self.lifespan < 0) then
		self:SetScript("OnUpdate", nil)
	end
end)

local function NewMediaResourceRegistered(event, mediatype, key)
	mediaResourceFrame.newMediaRegistered = 1
	mediaResourceFrame.timer = 0
end

function Healcsi:OnInitialize()
	-- Code that you want to run when the addon is first loaded goes here.

	--[[
		local textureBL = self.UIParent:CreateTexture(nil, "BACKGROUND")
		textureBL:SetPoint("BOTTOMLEFT", self.UIParent, "BOTTOMLEFT", 0, 0)
		textureBL:SetPoint("TOPRIGHT", self.UIParent, "BOTTOMLEFT", 20, 20)
		self:DecorateTexture(textureBL, "background", "Solid", "Orange", 0.7)

		local textureTR = self.UIParent:CreateTexture(nil, "BACKGROUND")
		textureTR:SetPoint("TOPRIGHT", self.UIParent, "TOPRIGHT", 0, 0)
		textureTR:SetPoint("BOTTOMLEFT", self.UIParent, "TOPRIGHT", -20, -20)
		self:DecorateTexture(textureTR, "background", "Solid", "Red", 0.7)
	]]

	for _, mod in pairs(Healcsi.Modules) do
		self.defaults.profile[mod.key] = mod.defaults
	end

	if (Healcsi:TableCount(Healcsi.Modules) > 1) then -- Sorting modules, could be optimised...
		for o_key, o_mod in pairs(Healcsi.Modules) do
			for i_key, i_mod in pairs(Healcsi.Modules) do
				if (o_mod.id < i_mod.id) then
					local tmp = Healcsi.Modules[o_key]
					Healcsi.Modules[o_key] = Healcsi.Modules[i_key]
					Healcsi.Modules[i_key] = tmp
				end
			end
		end
	end

	self.db = LibStub("AceDB-3.0"):New("HealcsiDB", self.defaults, "Default")

	if (Healcsi.DualSpec) then
		Healcsi.DualSpec:EnhanceDatabase(self.db, "HealcsiDB")
		Healcsi.DualSpec:EnhanceOptions(LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), self.db)
	end

	self.db:RegisterCallback("OnProfileChanged", OnProfileCallback)
	self.db:RegisterCallback("OnProfileDeleted", OnProfileCallback)
	self.db:RegisterCallback("OnProfileCopied", OnProfileCallback)
	self.db:RegisterCallback("OnProfileReset", OnProfileCallback)
	self.db:RegisterCallback("OnNewProfile", OnProfileCallback)

	Healcsi.Media:RegisterCallback("LibSharedMedia_Registered", NewMediaResourceRegistered)

	for _, mod in pairs(Healcsi.Modules) do
		mod:Initialize()
	end

	local configWindow = Gui:Create("HealcsiWindow")
	configWindow:SetBgFile(Healcsi.db.global.configWindow.bgFile)

	Healcsi:RegisterEventCallback("EditorConfigChanged", function()
		configWindow:SetBgFile(Healcsi.db.global.configWindow.bgFile)
	end)

	local profileEditor = Healcsi:CreateProfileEditor()
	local modulesEditor = Healcsi:CreateModulesEditor()

	configWindow.profileEditor = profileEditor
	configWindow.modulesEditor = modulesEditor

	configWindow:AddChildren(profileEditor, modulesEditor)

	configWindow:SetCallback("MenuChanged", ConfigWindow_Changed)
	configWindow:SetCallback("OnShow", ConfigWindow_Changed)
	configWindow.content:SetScript("OnSizeChanged", ConfigWindow_SizeChanged)
	configWindow:Hide()

	-- we will implement our own Layout cause the default always wants to call :Show() on all child elements
	-- configWindow:PauseLayout()

	self.configWindow = configWindow
	configWindow.obj = self

	Healcsi.Console:RegisterChatCommand("healcsi", SlashProcessor)

	Healcsi:Print(L["Greeting"])

end

function Healcsi:OnEnable()
    -- Called when the addon is enabled

end

function Healcsi:OnDisable()
    -- Called when the addon is disabled
end