
local Healcsi = Healcsi

-- Healcsi:GetColor(name, alpha)
-- Healcsi:GetClassColor(name, alpha)
-- Healcsi:HSV2RGB(h, s, v)
-- Healcsi:RGB2HSV(r, g, b)
-- Healcsi:AddHSV2RGB(r, g, b, h, s, v, returnHsv)

Healcsi.Color = {
    ["White"] =			{243/255, 244/255, 246/255},
	["White2"] =		{154/255, 156/255, 162/255},
	["Gray"] =			{ 87/255,  96/255, 105/255},
	["Tapped"] =		{ 87/255,  96/255, 105/255},
	["Blue"] =			{ 51/255, 214/255, 255/255},
	["Blue2"] =			{ 31/255, 174/255, 255/255},
	["BlueBackground"] = {102/255, 147/255, 168/255},
	["BlueSelected"]   = {178/255, 221/255, 202/255},
	["BlueHovered"]    = {54/255, 60/255, 133/255},
	["BluePressed"]    = {69/255, 104/255, 186/255},
	["BlueSelector"]   = {138/255, 222/255, 212/255},
	["DarkBlue"]       = {32/255, 29/255, 80/255},
	["Green"] =			{192/255, 252/255, 226/255},
	["Green2"] =		{ 39/255, 172/255, 96/255},
	["Red"] =			{255/255,  29/255,  65/255},
	["Red2"] =			{255/255,  77/255, 105/255},
	["Yellow"] =		{253/255, 218/255,   4/255},
	["Orange"] =		{219/255, 140/255,  15/255},
	["Yellow3"] =		{205/255, 201/255,  125/255},
	["Black"] =			{ 23/255,  23/255,  23/255},
	["Black2"] =		{ 52/255, 48/255, 78/255},
	
	["Class"] = {
		["DEATHKNIGHT"] =	{255/255,  49/255,  88/255},		--{247/255,  56/255,  84/255}
		["DRUID"] =			{255/255, 139/255,  36/255},		--{255/255, 125/255,  10/255}
		["HUNTER"] =		{206/255, 248/255,  74/255},		--{148/255, 219/255, 125/255}
		["MAGE"] =			{ 46/255, 208/255, 255/255},		--{  0/255, 194/255, 255/255}
		["PALADIN"] =		{255/255,  82/255, 204/255},		--{255/255,  56/255, 133/255}
		["PRIEST"] =		{218/255, 254/255, 254/255},		--{204/255, 252/255, 252/255}
		["ROGUE"] =			{255/255, 246/255,  14/255},		--{255/255, 232/255,  51/255}
		["SHAMAN"] =		{ 68/255, 148/255, 255/255},		--{  0/255, 153/255, 153/255}
		["WARLOCK"] =		{162/255, 144/255, 255/255},		--{153/255, 120/255, 217/255}
		["WARRIOR"] =		{242/255, 194/255, 160/255},		--{230/255, 166/255, 115/255}
		["MONK"] =			{ 96/255, 255/255, 194/255},		--{ 96/255, 255/255, 194/255}
		["DEMONHUNTER"] = 	{ 163/255, 49/255, 201/255},        --{163/255,  49/255, 201/255}
	},
	["Power"] = {}
}

function Healcsi:ConvColorObj(color)
	return color.r or 0, color.g or 0, color.b or 0, color.a or 1
end

function Healcsi:GetColor(name, alpha)
	if (name == nil) or (name == "Class") or (self.Color[name] == nil) then
		return 1, 1, 1
	else
		local color = self.Color[name]
		return color[1], color[2], color[3], alpha or 1
	end
end

function Healcsi:GetClassColor(name, alpha)
	if (name == nil) or (self.Color["Class"][name] == nil) then
		return 1, 1, 1
	else
		local color = self.Color["Class"][name]
		return color[1], color[2], color[3], alpha or 1
	end
end

function Healcsi:ApplyHsvShade(r, g, b, a, shade)
	local h, s, v = Healcsi:RGB2HSV(r, g, b)

	h = h + shade.h
	if h > 360 then h = h - 360 end
	if h < 0 then h = h + 360 end

	s = s + shade.s
	if s > 1 then s = 1 end
	if s < 0 then s = 0 end
	
	v = v + shade.v
	if v > 1 then v = 1 end
	if v < 0 then v = 0 end

	local newR, newG, newB = Healcsi:HSV2RGB(h, s, v)

	return newR, newG, newB, a
end

function Healcsi:ApplyRgbShade(r, g, b, a, shade)
	local newR = r + shade.r
	local newG = g + shade.g
	local newB = b + shade.b
	local newA = (a or 1) + (shade.a or 1)
	return newR, newG, newB, newA
end

-- @param h        hue (0-360)
-- @param s        saturation (0.0-1.0)
-- @param V        value (0.0-1.0)
-- @return         an R, G, and B component of RGB
-- http://www.rapidtables.com/convert/color/hsv-to-rgb.htm
function Healcsi:HSV2RGB(h, s, v)
	local c = v * s
	local x = c * (1 - abs( ((h/60)%2) - 1 ))
	local m = v - c

	local r, g, b

	if (h < 60 or h == 360) then -- 360 would be missing otherwise... i think
		r, g, b = c, x, 0
	elseif (h < 120) then
		r, g, b = x, c, 0
	elseif (h < 180) then
		r, g, b = 0, c, x
	elseif (h < 240) then
		r, g, b = 0, x, c
	elseif (h < 300) then
		r, g, b = x, 0, c
	elseif (h < 360) then
		r, g, b = c, 0, x
	end

	return r+m, g+m, b+m
end

-- http://www.rapidtables.com/convert/color/rgb-to-hsv.htm
function Healcsi:RGB2HSV(r, g, b)
	r = r or 0
	g = g or 0
	b = b or 0
	local cmax = math.max(r, g, b)
	local cmin = math.min(r, g, b)
	local d = cmax - cmin

	local hue
	if d ==0 then
		hue = 0
	elseif cmax == r then
		hue = 60 * (((g-b)/d)%6)
	elseif cmax == g then
		hue = 60 * (((b-r)/d)+2)
	elseif cmax == b then
		hue = 60 * (((r-g)/d)+4)
	end

	local saturation
	if cmax == 0 then
		saturation = 0
	else
		saturation = d/cmax
	end

	local value = cmax

	return hue, saturation, value
end

function Healcsi:AddHSV2RGB(r, g, b, h, s, v, returnHsv)
	local origH, origS, origV = Healcsi:RGB2HSV(r, g, b)

	h = origH+h
	s = origS+s
	v = origV+v

	if h > 360 then h = 360 end
	if h < 0 then h = 0 end

	if s > 1 then s = 1 end
	if s < 0 then s = 0 end

	if v > 1 then v = 1 end
	if v < 0 then v = 0 end

	if (returnHsv) then
		return h, s, v
	end

	return Healcsi:HSV2RGB(h, s, v)
end