--[[-----------------------------------------------------------------------------
InlineGroup Container
Simple container widget that creates a visible "box" with an optional title.
-------------------------------------------------------------------------------]]
local Healcsi = Healcsi

local Type, Version = "Healcsi-InlineGroup", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local pairs = pairs

-- WoW APIs
local CreateFrame = CreateFrame

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		self:SetWidth(300)
		self:SetHeight(100)
		self:SetTitle("")
		self:SetMargin({0, 0, 0, 0})

		self:SetBackground({0, 0, 0, 0}, "Solid", "Black", 0)
		self:SetLineBackground(0, 0, "Solid", "Black", 0)
	end,

	["OnRelease"] = function(self)
		self.frame:SetParent(nil)
	end,

	["SetTitle"] = function(self,title)
		--self.titletext:SetText(title)
	end,

	["SetMargin"] = function(self, margin)

		margin = margin or {0, 0, 0, 0}

		local top     = margin[1] or 0
		local right   = margin[2] or 0
		local bottom  = margin[3] or 0
		local left    = margin[4] or 0

		self.content:ClearAllPoints()
		self.content:SetPoint("TOPLEFT", left, -top)
		self.content:SetPoint("BOTTOMRIGHT", -right, bottom)
		self.content.margin = margin
	end,

	["SetBackground"] = function(self, margin, textureName, colorName, alpha)
		Healcsi:SetPadding(self.frame.texture, self.frame, margin)
		Healcsi:DecorateTexture(self.frame.texture, "background", textureName, colorName, alpha)
	end,

	["SetLineBackground"] = function(self, leftWidth, rightWidth, textureName, colorName, alpha)
		Healcsi:SetLeftLine(self.frame.texture.left, self.frame, leftWidth)
		Healcsi:SetRightLine(self.frame.texture.right, self.frame, rightWidth)

		Healcsi:DecorateTexture(self.frame.texture.left, "background", textureName, colorName, alpha)
		Healcsi:DecorateTexture(self.frame.texture.right, "background", textureName, colorName, alpha)
	end,

	["LayoutFinished"] = function(self, width, height)
		if self.noAutoHeight then return end
		local newHeight = 0
		if (height) then
			newHeight = height + (self.content.margin[1] or 0) + (self.content.margin[3] or 0)
		else
			newHeight = (self.content.margin[1] or 0) + (self.content.margin[3] or 0)
		end
		self:SetHeight(newHeight) -- no + 40 !!@@ wtf ffs
	end,

	["OnWidthSet"] = function(self, width)
		local content = self.content
		local contentwidth = width - 20
		if contentwidth < 0 then
			contentwidth = 0
		end
		content:SetWidth(contentwidth)
		content.width = contentwidth
	end,

	["OnHeightSet"] = function(self, height)
		local content = self.content
		local contentheight = height - 20
		if contentheight < 0 then
			contentheight = 0
		end
		content:SetHeight(contentheight)
		content.height = contentheight
	end

}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]

local function Constructor()
	local frame = CreateFrame("Frame", nil, Healcsi.UIParent)
	frame:SetFrameStrata("FULLSCREEN_DIALOG")

	--[[
	local titletext = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	titletext:SetPoint("TOPLEFT", 14, 0)
	titletext:SetPoint("TOPRIGHT", -14, 0)
	titletext:SetJustifyH("LEFT")
	titletext:SetHeight(18)
	]]

	frame.texture = Healcsi:CreateTextureFull(frame, "BACKGROUND", {5, 5, 5, 5})
	Healcsi:DecorateTexture(frame.texture, "background", "Solid", "DarkBlue", 0)

	frame.texture.left = Healcsi:CreateLeftLineTexture(frame, "ARTWORK", 0)
	Healcsi:DecorateTexture(frame.texture.left, "background", "Solid", "DarkBlue", 0)

	frame.texture.right = Healcsi:CreateRightLineTexture(frame, "ARTWORK", 0)
	Healcsi:DecorateTexture(frame.texture.right, "background", "Solid", "DarkBlue", 0)

	--Container Support
	local content = CreateFrame("Frame", nil, frame)
	content:SetPoint("TOPLEFT", 0, 0)
	content:SetPoint("BOTTOMRIGHT", 0, 0)
	content.margin = 0

	local widget = {
		frame     = frame,
		content   = content,
		--titletext = titletext,
		type      = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end

	return AceGUI:RegisterAsContainer(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
