--[[-----------------------------------------------------------------------------
Healcsi Modules Selector
-------------------------------------------------------------------------------]]
local Healcsi      = Healcsi
local Gui          = Healcsi.Gui
local TopParent    = Healcsi.UIParent
local HiddenParent = Healcsi.HiddenParent

-- Lua APIs
local min, max, floor = math.min, math.max, math.floor
local select, pairs, ipairs, type = select, pairs, ipairs, type
local tsort = table.sort

-- WoW APIs
local CreateFrame = CreateFrame

local function fixlevels(parent,...)
	local i = 1
	local child = select(i, ...)
	while child do
		child:SetFrameLevel(parent:GetFrameLevel()+1)
		fixlevels(child, child:GetChildren())
		i = i + 1
		child = select(i, ...)
	end
end

local function fixstrata(strata, parent, ...)
	local i = 1
	local child = select(i, ...)
	parent:SetFrameStrata(strata)
	while child do
		fixstrata(strata, child, child:GetChildren())
		i = i + 1
		child = select(i, ...)
	end
end



local IBLib = LibStub("AceGUI-3.0-DropDown-ItemBase")
local ItemBase = IBLib:GetItemBase()






do
	local widgetType = "Healcsi-Dropdown-Item-Header"
	local widgetVersion = 1
	
	local function OnEnter(this)
		local self = this.obj
		self:Fire("OnEnter")
		
		if self.specialOnEnter then
			self.specialOnEnter(self)
		end
	end
	
	local function OnLeave(this)
		local self = this.obj
		self:Fire("OnLeave")
		
		if self.specialOnLeave then
			self.specialOnLeave(self)
		end
	end
	
	-- exported, override
	local function SetDisabled(self, disabled)
		ItemBase.SetDisabled(self, disabled)
		if not disabled then
			self.text:SetTextColor(Healcsi:GetColor("BlueSelector", 1))
		end
	end
	
	local function Constructor()
		local self = ItemBase.Create(widgetType)
		
		self.SetDisabled = SetDisabled
		
		self.frame:SetScript("OnEnter", OnEnter)
		self.frame:SetScript("OnLeave", OnLeave)
		
		self.text:SetTextColor(Healcsi:GetColor("BlueSelector", 1))
		
		Gui:RegisterAsWidget(self)
		return self
	end
	
	Gui:RegisterWidgetType(widgetType, Constructor, widgetVersion + ItemBase.version)
end



-- Item: Separator
-- A single line to separate items
do
	local widgetType = "Healcsi-Dropdown-Item-Separator"
	local widgetVersion = 1
	
	-- exported, override
	local function SetDisabled(self, disabled)
		ItemBase.SetDisabled(self, disabled)
		self.useHighlight = false
	end
		
	local function Constructor()
		local self = ItemBase.Create(widgetType)
		
		self.SetDisabled = SetDisabled
		
		local line = self.frame:CreateTexture(nil, "OVERLAY")
		line:SetHeight(2)
		line:SetColorTexture(.8, .8, .8)
		line:SetPoint("LEFT", self.frame, "LEFT", 10, 0)
		line:SetPoint("RIGHT", self.frame, "RIGHT", -10, 0)
		
		self.text:Hide()
		
		self.useHighlight = false
		
		Gui:RegisterAsWidget(self)
		return self
	end
	
	Gui:RegisterWidgetType(widgetType, Constructor, widgetVersion + ItemBase.version)
end






-- Item: Toggle
-- Some sort of checkbox for dropdown menus.
-- Does not close the pullout on click.
do
	local widgetType = "Healcsi-Dropdown-Learnt-Spell"
	local widgetVersion = 1
	
	local function UpdateToggle(self)
		if self.value then
			self.check:Show()
		else
			self.check:Hide()
		end
	end
	
	local function OnRelease(self)
		ItemBase.OnRelease(self)
		self:SetValue(nil)
	end

	local function SetText(self, text)
		local name, rank, icon, castingTime, minRange, maxRange, spellID = GetSpellInfo(text)
		self.icon:SetTexture(icon or nil)
		self.text:SetText(name or "")
	end
	
	local function Frame_OnClick(this, button)
		local self = this.obj
		if self.disabled then return end
		self.value = not self.value
		if self.value then
			--PlaySound("igMainMenuOptionCheckBoxOn")
		else
			--PlaySound("igMainMenuOptionCheckBoxOff")
		end
		UpdateToggle(self)
		self:Fire("OnValueChanged", self.value)
	end
	
	-- exported
	local function SetValue(self, value)
		self.value = value
		UpdateToggle(self)
	end
	
	-- exported
	local function GetValue(self)
		return self.value
	end
	
	local function Constructor()
		local self = ItemBase.Create(widgetType)
		
		self.frame:SetScript("OnClick", Frame_OnClick)
		
		self.SetValue  = SetValue
		self.GetValue  = GetValue
		self.OnRelease = OnRelease
		self.SetText   = SetText

		local icon = self.frame:CreateTexture(nil, "OVERLAY")
		self.icon = icon

		icon:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 20, 0)
		icon:SetSize(self.frame:GetHeight(), self.frame:GetHeight())

		self.text:ClearAllPoints()
		self.text:SetPoint("LEFT", self.frame, "LEFT", 25 + self.frame:GetHeight(), 0)
		
		Gui:RegisterAsWidget(self)
		return self
	end
	
	Gui:RegisterWidgetType(widgetType, Constructor, widgetVersion + ItemBase.version)
end




-- Item: Toggle
-- Some sort of checkbox for dropdown menus.
-- Does not close the pullout on click.
do
	local widgetType = "Healcsi-Dropdown-Unlearnt-Spell"
	local widgetVersion = 1
	
	local function UpdateToggle(self)
		if self.value then
			self.check:Show()
		else
			self.check:Hide()
		end
	end
	
	local function OnRelease(self)
		ItemBase.OnRelease(self)
		self:SetValue(nil)
	end

	local function SetText(self, text)
		local name, rank, icon, castingTime, minRange, maxRange, spellID = GetSpellInfo(text)
		self.icon:SetTexture(icon or nil)
		self.text:SetText(name or "")
	end

	local function Frame_OnClick(this, button)
		local self = this.obj
		if self.disabled then return end
		self.value = not self.value
		if self.value then
			--PlaySound("igMainMenuOptionCheckBoxOn")
		else
			--PlaySound("igMainMenuOptionCheckBoxOff")
		end
		UpdateToggle(self)
		self:Fire("OnValueChanged", self.value)
	end
	
	-- exported
	local function SetValue(self, value)
		self.value = value
		UpdateToggle(self)
	end
	
	-- exported
	local function GetValue(self)
		return self.value
	end
	
	local function Constructor()
		local self = ItemBase.Create(widgetType)

		self.frame:SetScript("OnClick", Frame_OnClick)
		
		self.SetValue  = SetValue
		self.GetValue  = GetValue
		self.OnRelease = OnRelease
		self.SetText   = SetText

		local icon = self.frame:CreateTexture(nil, "OVERLAY")
		self.icon = icon

		icon:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 20, 0)
		icon:SetSize(self.frame:GetHeight(), self.frame:GetHeight())
		self.icon:SetVertexColor(0.6, 0.6, 0.6, 0.6)

		self.text:ClearAllPoints()
		self.text:SetPoint("LEFT", self.frame, "LEFT", 25 + self.frame:GetHeight(), 0)
		-- self.text:SetTextColor(0.6, 0.6, 0.6, 0.8)
		
		Gui:RegisterAsWidget(self)
		return self
	end
	
	Gui:RegisterWidgetType(widgetType, Constructor, widgetVersion + ItemBase.version)
end










do
	local widgetType = "Healcsi-ClickCast-Dropdown-Pullout"
	local widgetVersion = 1

	--[[ Static data ]]--

	local defaultWidth = 200
	local defaultMaxHeight = 400

	--[[ UI Event Handlers ]]--

	-- HACK: This should be no part of the pullout, but there
	--       is no other 'clean' way to response to any item-OnEnter
	--       Used to close Submenus when an other item is entered
	local function OnEnter(item)
		local self = item.pullout
		for k, v in ipairs(self.items) do
			if v.CloseMenu and v ~= item then
				v:CloseMenu()
			end
		end
	end

	-- See the note in Constructor() for each scroll related function
	local function OnMouseWheel(this, value)
		this.obj:MoveScroll(value)
	end
	
	local function OnScrollValueChanged(this, value)
		this.obj:SetScroll(value)
	end
	
	local function OnSizeChanged(this)
		this.obj:FixScroll()
	end

	--[[ Exported methods ]]--
	
	-- exported
	local function SetScroll(self, value)
		local status = self.scrollStatus
		local frame, child = self.scrollFrame, self.itemFrame
		local height, viewheight = frame:GetHeight(), child:GetHeight()

		local offset
		if height > viewheight then
			offset = 0
		else
			offset = floor((viewheight - height) / 1000 * value)
		end
		child:ClearAllPoints()
		child:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, offset)
		child:SetPoint("TOPRIGHT", frame, "TOPRIGHT", self.slider:IsShown() and -12 or 0, offset)
		status.offset = offset
		status.scrollvalue = value		
	end
	
	-- exported
	local function MoveScroll(self, value)
		local status = self.scrollStatus
		local frame, child = self.scrollFrame, self.itemFrame
		local height, viewheight = frame:GetHeight(), child:GetHeight()

		if height > viewheight then
			self.slider:Hide()
		else
			self.slider:Show()
			local diff = height - viewheight
			local delta = 1
			if value < 0 then
				delta = -1
			end
			self.slider:SetValue(min(max(status.scrollvalue + delta*(1000/(diff/45)),0), 1000))
		end
	end
	
	-- exported
	local function FixScroll(self)
		local status = self.scrollStatus
		local frame, child = self.scrollFrame, self.itemFrame
		local height, viewheight = frame:GetHeight(), child:GetHeight()
		local offset = status.offset or 0

		if viewheight < height then
			self.slider:Hide()
			child:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, offset)
			self.slider:SetValue(0)
		else
			self.slider:Show()			
			local value = (offset / (viewheight - height) * 1000)
			if value > 1000 then value = 1000 end
			self.slider:SetValue(value)
			self:SetScroll(value)
			if value < 1000 then
				child:ClearAllPoints()
				child:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, offset)
				child:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -12, offset)
				status.offset = offset
			end
		end
	end

	-- exported, AceGUI callback
	local function OnAcquire(self)
		self.frame:SetParent(TopParent)
		--self.itemFrame:SetToplevel(true)
	end
	
	-- exported, AceGUI callback
	local function OnRelease(self)
		self:Clear()
		self.frame:ClearAllPoints()
		self.frame:Hide()
	end

	-- exported
	local function AddItem(self, item)
		self.items[#self.items + 1] = item
		
		local h = #self.items * 16
		self.itemFrame:SetHeight(h)
		self.frame:SetHeight(min(h + 34, self.maxHeight)) -- +34: 20 for scrollFrame placement (10 offset) and +14 for item placement
		
		item.frame:SetPoint("LEFT", self.itemFrame, "LEFT")
		item.frame:SetPoint("RIGHT", self.itemFrame, "RIGHT")
		
		if (item.text and self.itemColor) then
			local color = self.itemColor
			item.text:SetVertexColor(color.r, color.g, color.b, color.a or 1)
			item.text:SetShadowColor(0, 0, 0, 0)
		end

		if (item.text and self.itemFont) then
			item.text:SetFont(Healcsi:GetMedia("font", self.itemFont) or self.itemFont, self.itemHeight or 13, self.itemFlags)
		end

		item:SetPullout(self)
		item:SetOnEnter(OnEnter)
	end

	-- exported
	local function Open(self, point, relFrame, relPoint, x, y)		
		local items = self.items
		local frame = self.frame
		local itemFrame = self.itemFrame
		
		frame:SetPoint(point, relFrame, relPoint, x, y)

				
		local height = 8
		for i, item in pairs(items) do
			if i == 1 then
				item:SetPoint("TOP", itemFrame, "TOP", 0, -2)
			else
				item:SetPoint("TOP", items[i-1].frame, "BOTTOM", 0, 1)
			end
			
			item:SetHeight(16)
			item:Show()
			
			height = height + 16
		end
		itemFrame:SetHeight(height)
		fixstrata("TOOLTIP", frame, frame:GetChildren())
		frame:Show()
		self:Fire("OnOpen")
	end

	-- exported
	local function Close(self)
		self.frame:Hide()
		self:Fire("OnClose")
	end	
	
	-- exported
	local function Clear(self)
		local items = self.items
		for i, item in pairs(items) do
			Gui:Release(item)
			items[i] = nil
		end
	end	

	-- exported
	local function IterateItems(self)
		return ipairs(self.items)
	end
	
	-- exported
	local function SetHideOnLeave(self, val)
		self.hideOnLeave = val
	end

	-- exported
	local function SetMaxHeight(self, height)
		self.maxHeight = height or defaultMaxHeight
		if self.frame:GetHeight() > height then
			self.frame:SetHeight(height)
		elseif (self.itemFrame:GetHeight() + 34) < height then
			self.frame:SetHeight(self.itemFrame:GetHeight() + 34) -- see :AddItem
		end
	end

	-- exported
	local function GetRightBorderWidth(self)
		return 6 + (self.slider:IsShown() and 12 or 0)
	end
	
	-- exported
	local function GetLeftBorderWidth(self)
		return 6
	end

	local function SetBackdrop(self, backdrop)
		self.frame:SetBackdrop(backdrop)
	end

	local function SetBackdropColor(self, r, g, b, a)
		self.frame:SetBackdropColor(r, g, b, a or 1)
	end

	local function SetFont(self, font, height, flags)
		self.itemFont = font
		self.itemHeight = height
		self.itemFlags = flags
		local items = self.items
		for i, item in pairs(items) do
			if (item.text) then
				item.text:SetFont(Healcsi:GetMedia("font", font) or font, height or 13, flags)
			end
		end
	end

	local function SetFontColor(self, r, g, b, a)
		self.itemColor = self.itemColor or {}
		self.itemColor.r = r
		self.itemColor.g = g
		self.itemColor.b = b
		self.itemColor.a = a or 1
		local items = self.items
		for i, item in pairs(items) do
			if (item.text) then
				item.text:SetVertexColor(r, g, b, a or 1)
				item.text:SetShadowColor(0, 0, 0, 0)
			end
		end
	end

	local function SetScrollbar(self, backdrop, width, r, g, b, a)
		self.slider:SetBackdrop(backdrop)
		self.slider:SetBackdropColor(r, g, b, a or 1)
		self.slider:SetWidth(width)
	end

	local function SetScrollbarThumb(self, thumbTexture, width, height, r, g, b, a)
		local thumb = self.slider.thumb

		thumb:SetSize(width, height)
		thumb:SetTexture(Healcsi:GetMedia("statusbar", thumbTexture) or thumbTexture)
		thumb:SetVertexColor(r, g, b, a or 1)
	end


	--[[ Constructor ]]--
	
	local function Constructor()
		local count = Gui:GetNextWidgetNum(widgetType)
		local frame = CreateFrame("Frame", "AceGUI30HealcsiClickCastDropdownPullout"..count, TopParent)
		local self = {}
		self.count = count
		self.type = widgetType
		self.frame = frame
		frame.obj = self

		self.OnAcquire = OnAcquire
		self.OnRelease = OnRelease

		self.SetBackdrop = SetBackdrop
		self.SetBackdropColor = SetBackdropColor

		self.SetFont = SetFont
		self.SetFontColor = SetFontColor

		self.SetScrollbar = SetScrollbar
		self.SetScrollbarThumb = SetScrollbarThumb

		self.AddItem = AddItem
		self.Open    = Open
		self.Close   = Close
		self.Clear   = Clear
		self.IterateItems = IterateItems
		self.SetHideOnLeave = SetHideOnLeave

		self.SetScroll  = SetScroll
		self.MoveScroll = MoveScroll
		self.FixScroll  = FixScroll
		
		self.SetMaxHeight = SetMaxHeight
		self.GetRightBorderWidth = GetRightBorderWidth
		self.GetLeftBorderWidth = GetLeftBorderWidth
		
		self.items = {}
		
		self.scrollStatus = {
			scrollvalue = 0,
		}

		self.maxHeight = defaultMaxHeight

		frame:SetFrameStrata("FULLSCREEN_DIALOG")
		frame:SetClampedToScreen(true)
		frame:SetWidth(defaultWidth)
		frame:SetHeight(self.maxHeight)	

		-- NOTE: The whole scroll frame code is copied from the AceGUI-3.0 widget ScrollFrame
		local scrollFrame = CreateFrame("ScrollFrame", nil, frame)
		local itemFrame = CreateFrame("Frame", nil, scrollFrame)

		self.scrollFrame = scrollFrame
		self.itemFrame = itemFrame

		scrollFrame.obj = self
		itemFrame.obj = self

		local slider = CreateFrame("Slider", "AceGUI30HealcsiClickCastPulloutScrollbar"..count, scrollFrame)
		slider:SetOrientation("VERTICAL")
		slider:SetHitRectInsets(0, 0, -10, 0)
		slider:SetWidth(8)
		slider:SetFrameStrata("FULLSCREEN_DIALOG")
		self.slider = slider
		slider.obj = self

		local thumb = slider:CreateTexture(nil, "OVERLAY")
		thumb:SetSize(8, 10)
		slider:SetThumbTexture(thumb)
		slider.thumb = thumb

		scrollFrame:SetScrollChild(itemFrame)
		scrollFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 6, -12)
		scrollFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -6, 12)
		scrollFrame:EnableMouseWheel(true)
		scrollFrame:SetScript("OnMouseWheel", OnMouseWheel)
		scrollFrame:SetScript("OnSizeChanged", OnSizeChanged)
		scrollFrame:SetToplevel(true)
		scrollFrame:SetFrameStrata("FULLSCREEN_DIALOG")
		
		itemFrame:SetPoint("TOPLEFT", scrollFrame, "TOPLEFT", 0, 0)
		itemFrame:SetPoint("TOPRIGHT", scrollFrame, "TOPRIGHT", -12, 0)
		itemFrame:SetHeight(400)
		itemFrame:SetToplevel(true)
		itemFrame:SetFrameStrata("FULLSCREEN_DIALOG")
		
		slider:SetPoint("TOPLEFT", scrollFrame, "TOPRIGHT", -16, 0)
		slider:SetPoint("BOTTOMLEFT", scrollFrame, "BOTTOMRIGHT", -16, 0)
		slider:SetScript("OnValueChanged", OnScrollValueChanged)
		slider:SetMinMaxValues(0, 1000)
		slider:SetValueStep(1)
		slider:SetValue(0)
		
		scrollFrame:Show()
		itemFrame:Show()
		slider:Hide()
						
		self:FixScroll()

		Gui:RegisterAsWidget(self)
		return self
	end

	Gui:RegisterWidgetType(widgetType, Constructor, widgetVersion)

end

do
	local widgetType = "Healcsi-ClickCastDropdown"
	local widgetVersion = 1

	--[[ Static data ]]--

	--[[ UI event handler ]]--

	local function Control_OnEnter(this)
		this.obj.button:LockHighlight()
		this.obj:Fire("OnEnter")
	end
	
	local function Control_OnLeave(this)
		this.obj.button:UnlockHighlight()
		this.obj:Fire("OnLeave")
	end

	local function Dropdown_OnHide(this)
		local self = this.obj
		if self.open then
			self.pullout:Close()
		end
	end

	local function Dropdown_TogglePullout(this)
		local self = this.obj
		if self.open then
			self.open = nil
			self.pullout:Close()
			Gui:ClearFocus()
		else
			self.open = true
			self.pullout:SetWidth(self.pulloutWidth or self.frame:GetWidth())
			self.pullout:Open("TOPLEFT", self.frame, "BOTTOMLEFT", 0, 2) -- self.label:IsShown() and -2 or 0
			Gui:SetFocus(self)
		end
	end

	local function OnPulloutOpen(this)
		local self = this.userdata.obj
		local value = self.value
		
		if not self.multiselect then
			for i, item in this:IterateItems() do
				if (item.SetValue) then
					item:SetValue(item.userdata.value == value)
				end
			end
		end
		
		self.open = true
		self:Fire("OnOpened")
	end

	local function OnPulloutClose(this)
		local self = this.userdata.obj
		self.open = nil
		self:Fire("OnClosed")
	end

	local function ShowMultiText(self)
		local text
		for i, widget in self.pullout:IterateItems() do
			if widget.type == "Dropdown-Item-Toggle" then
				if widget:GetValue() then
					if text then
						text = text..", "..widget:GetText()
					else
						text = widget:GetText()
					end
				end
			end
		end
		self:SetText(text)
	end

	local function OnItemValueChanged(this, event, checked)
		local self = this.userdata.obj
		
		if self.multiselect then
			self:Fire("OnValueChanged", this.userdata.value, checked)
			ShowMultiText(self)
		else
			if checked then
				self:SetValue(this.userdata.value)
				self:Fire("OnValueChanged", this.userdata.value)
			else
				this:SetValue(true)
			end
			if self.open then	
				self.pullout:Close()
			end
		end
	end

	--[[ Exported methods ]]--


	local function SetSelectorTexture(self, texture)
		self.selector:SetTexture(Healcsi:GetMedia("statusbar", texture) or texture)
	end

	local function SetSelectorTextureColor(self, r, g, b, a)
		self.selector:SetVertexColor(r or 0, g or 0, b or 0, a or 1)
	end

	local function SetSelectorFont(self, font, height, flags)
		self.text:SetFont(Healcsi:GetFont(font), height or 13, flags)
	end

	local function SetSelectorFontColor(self, r, g, b, a)
		self.text:SetTextColor(r or 0, g or 0, b or 0, a or 1)
	end

	local function SetLabelFont(self, font, height, flags)
		self.label:SetFont(Healcsi:GetFont(font), height or 14, flags)
	end

	local function SetLabelFontColor(self, r, g, b, a)
		self.label:SetVertexColor(r or 0, g or 0, b or 0, a or 1)
	end

	local function SetLabelShadowColor(self, r, g, b, a)
		self.label:SetShadowColor(r or 0, g or 0, b or 0, a or 1)
	end

	local function SetLabelShadowOffset(self, x, y)
		self.label:SetShadowOffset(x or 0, y or 0)
	end

	-- exported, AceGUI callback
	local function OnAcquire(self)
		local pullout = Gui:Create("Healcsi-ClickCast-Dropdown-Pullout")
		self.pullout = pullout
		pullout.userdata.obj = self
		pullout:SetCallback("OnClose", OnPulloutClose)
		pullout:SetCallback("OnOpen", OnPulloutOpen)
		self.pullout.frame:SetFrameLevel(self.frame:GetFrameLevel() + 1)
		fixlevels(self.pullout.frame, self.pullout.frame:GetChildren())

		self:SetLabelFont("Friz Quadrata TT")
		self:SetLabelFontColor(Healcsi:GetColor("Black", 1))

		self:SetLabelShadowColor(Healcsi:GetColor("DarkBlue", 0.5))
		self:SetLabelShadowOffset(1, -1)

		self:SetSelectorTexture("LiteStep")
		self:SetSelectorTextureColor(Healcsi:GetColor("DarkBlue", 0.9))

		self:SetSelectorFont("Friz Quadrata TT")
		self:SetSelectorFontColor(Healcsi:GetColor("White", 0.9))

		Healcsi:DecorateTexture(self.button.normalTexture,    "background", "DownArrowSmall", "BlueSelector", 1)
		Healcsi:DecorateTexture(self.button.disabledTexture,  "background", "DownArrowSmall", "BlueSelector", 0.3)
		Healcsi:DecorateTexture(self.button.pushedTexture,    "background", "DownArrowSmall", "Yellow", 1)
		Healcsi:DecorateTexture(self.button.highlightTexture, "background", "DownArrowSmall", "Blue", 1)


		local backdrop = {
			bgFile = Healcsi:GetMedia("background", "Solid"),
			tileSize = 32,
			tile = true,
			insets = { left = 6, right = 7, top = 6, bottom = 4 }
		}

		pullout:SetBackdrop(backdrop)
		pullout:SetBackdropColor(Healcsi:GetColor("DarkBlue", 1))

		local sliderBackdrop = {
			bgFile = Healcsi:GetMedia("background", "Solid"),
			tileSize = 8,
			insets = { left = 3, right = 3, top = 3, bottom = 3 }
		}

		pullout:SetScrollbar(sliderBackdrop, 8, Healcsi:GetColor("BlueSelector", 1))
		pullout:SetScrollbarThumb(Healcsi:GetMedia("background", "Solid"), 10, 32, Healcsi:GetColor("BlueSelector", 1))

		pullout:SetFont("Friz Quadrata TT")
		-- pullout:SetFontColor(Healcsi:GetColor("White", 1))
		
		self:SetHeight(44)
		self:SetWidth(200)
		self:SetLabel()
		self:SetPulloutWidth(nil)
	end

	-- exported, AceGUI callback
	local function OnRelease(self)
		if self.open then
			self.pullout:Close()
		end
		Gui:Release(self.pullout)
		self.pullout = nil
		
		self:SetText("")
		self:SetDisabled(false)
		self:SetMultiselect(false)
		
		self.value = nil
		self.list = nil
		self.open = nil
		self.hasClose = nil
		
		self.frame:ClearAllPoints()
		self.frame:Hide()
	end

	-- exported
	local function SetDisabled(self, disabled)
		self.disabled = disabled
		if disabled then
			--self.text:SetTextColor(0.5,0.5,0.5)
			self:SetSelectorTextureColor(Healcsi:GetColor("Gray", 0.9))
			self.button:Disable()
			self.button_cover:Disable()
			--self.label:SetTextColor(0.5,0.5,0.5)
			self:SetLabelFontColor(Healcsi:GetColor("Gray", 1))
		else
			self.button:Enable()
			self.button_cover:Enable()
			--self.label:SetTextColor(1,.82,0)
			self:SetLabelFontColor(Healcsi:GetColor("Black", 1))
			--self.text:SetTextColor(1,1,1)
			self:SetSelectorTextureColor(Healcsi:GetColor("DarkBlue", 0.9))
		end
	end

	-- exported
	local function ClearFocus(self)
		if self.open then
			self.pullout:Close()
		end
	end

	-- exported
	local function SetText(self, text)
		self.text:SetText(text or "")
	end

	-- exported
	local function SetLabel(self, text)
		if text and text ~= "" then
			self.label:SetText(text)
			self.label:Show()
			self.dropdown:SetPoint("TOPLEFT",self.frame,"TOPLEFT",-15,-20) -- -15, -16
			self:SetHeight(46)
			self.alignoffset = 26
		else
			self.label:SetText("")
			self.label:Hide()
			self.dropdown:SetPoint("TOPLEFT",self.frame,"TOPLEFT",-15,0)
			self:SetHeight(26)
			self.alignoffset = 12
		end
	end

	-- exported
	local function SetValue(self, value)
		if self.list then
			self:SetText(self.list[value] or "")
		end
		self.value = value
	end

	-- exported
	local function GetValue(self)
		return self.value
	end
	
	-- exported
	local function SetItemValue(self, item, value)
		if not self.multiselect then return end
		for i, widget in self.pullout:IterateItems() do
			if widget.userdata.value == item then
				if widget.SetValue then
					widget:SetValue(value)
				end
			end
		end
		ShowMultiText(self)
	end
	
	-- exported
	local function SetItemDisabled(self, item, disabled)
		for i, widget in self.pullout:IterateItems() do
			if widget.userdata.value == item then
				widget:SetDisabled(disabled)
			end
		end
	end

	local function AddListItem(self, value, text, itemType)
		if not itemType then itemType = "Dropdown-Item-Toggle" end
		local exists = Gui:GetWidgetVersion(itemType)
		if not exists then error(("The given item type, %q, does not exist within AceGUI-3.0"):format(tostring(itemType)), 2) end

		local item = Gui:Create(itemType)
		item:SetText(text)
		item.userdata.obj = self
		item.userdata.value = value
		item:SetCallback("OnValueChanged", OnItemValueChanged)
		self.pullout:AddItem(item)
	end

	local function AddCloseButton(self)
		if not self.hasClose then
			local close = Gui:Create("Dropdown-Item-Execute")
			close:SetText(CLOSE)
			self.pullout:AddItem(close)
			self.hasClose = true
		end
	end

	-- exported
	local sortlist = {}
	local function SetList(self, list)
		--self.list = list
		self.pullout:Clear()
		self.hasClose = nil
		if not list then return end

		local goodList = {}
		local action = {}
		local builtin = {}
		local custom = {}

		for key, value in pairs(list) do
			local order, section, name = string.match(key, "(%d+)-(%d+)-(.+)")
			if (section == "1") then
				action[#action + 1] = key
			elseif (section == "2") then
				builtin[#builtin + 1] = key
			elseif (section == "3") then
				custom[#custom + 1] = key
			end
		end

		tsort(action)
		tsort(builtin)
		tsort(custom)

		AddListItem(self, "header1", "Default Actions", "Healcsi-Dropdown-Item-Header")
		AddListItem(self, "header1-bottom", "", "Healcsi-Dropdown-Item-Separator")

		for i = 1, #(action) do
			goodList[action[i]] = list[action[i]]
			AddListItem(self, action[i], list[action[i]], itemType)
			action[i] = nil
		end

		AddListItem(self, "header2-top", "", "Healcsi-Dropdown-Item-Header")
		AddListItem(self, "header2", "Built-in Actions", "Healcsi-Dropdown-Item-Header")
		AddListItem(self, "header2-bottom", "", "Healcsi-Dropdown-Item-Separator")

		for i = 1, #(builtin) do
			local spell = list[builtin[i]]
			local spellInfo, state = string.match(spell, "(.+)_(.+)")

			if (state == "learnt") then
				AddListItem(self, builtin[i], spellInfo, "Healcsi-Dropdown-Learnt-Spell")
				goodList[builtin[i]] = spellInfo
			else
				AddListItem(self, builtin[i], spellInfo, "Healcsi-Dropdown-Unlearnt-Spell")
				local name, _, _, _, _, _, _ = GetSpellInfo(spellInfo)
				goodList[builtin[i]] = name .. " (Unlearnt)"
			end
			
			builtin[i] = nil
		end

		AddListItem(self, "header3-top", "", "Healcsi-Dropdown-Item-Header")
		AddListItem(self, "header3", "Custom Actions", "Healcsi-Dropdown-Item-Header")
		AddListItem(self, "header3-bottom", "", "Healcsi-Dropdown-Item-Separator")

		for i = 1, #(custom) do
			goodList[custom[i]] = list[custom[i]]
			AddListItem(self, custom[i], list[custom[i]], itemType)
			custom[i] = nil
		end

		self.list = goodList
	
		if self.multiselect then
			ShowMultiText(self)
			AddCloseButton(self)
		end
	end
	
	-- exported
	local function AddItem(self, value, text, itemType)
		if self.list then
			self.list[value] = text
			AddListItem(self, value, text, itemType)
		end
	end
	
	-- exported
	local function SetMultiselect(self, multi)
		self.multiselect = multi
		if multi then
			ShowMultiText(self)
			AddCloseButton(self)
		end
	end
	
	-- exported
	local function GetMultiselect(self)
		return self.multiselect
	end
	
	local function SetPulloutWidth(self, width)
		self.pulloutWidth = width
	end

	--[[ Constructor ]]--

	local function Constructor()
		local count = Gui:GetNextWidgetNum(widgetType)
		local frame = CreateFrame("Frame", nil, TopParent)
		local dropdown = CreateFrame("Frame", "AceGUI30HealcsiClickCastDropDownBox"..count, frame, "UIDropDownMenuTemplate")

		local self = {}
		self.type = widgetType
		self.frame = frame
		self.dropdown = dropdown
		self.count = count
		frame.obj = self
		dropdown.obj = self

		self.OnRelease   = OnRelease
		self.OnAcquire   = OnAcquire
		
		self.ClearFocus  = ClearFocus

		self.SetSelectorTexture = SetSelectorTexture
		self.SetSelectorTextureColor = SetSelectorTextureColor

		self.SetSelectorFont = SetSelectorFont
		self.SetSelectorFontColor = SetSelectorFontColor

		self.SetLabelFont = SetLabelFont
		self.SetLabelFontColor = SetLabelFontColor
		self.SetLabelShadowColor = SetLabelShadowColor
		self.SetLabelShadowOffset = SetLabelShadowOffset

		self.SetText     = SetText
		self.SetValue    = SetValue
		self.GetValue    = GetValue
		self.SetList     = SetList
		self.SetLabel    = SetLabel
		self.SetDisabled = SetDisabled
		self.AddItem     = AddItem
		self.SetMultiselect = SetMultiselect
		self.GetMultiselect = GetMultiselect
		self.SetItemValue = SetItemValue
		self.SetItemDisabled = SetItemDisabled
		self.SetPulloutWidth = SetPulloutWidth

		self.alignoffset = 26

		frame:SetScript("OnHide",Dropdown_OnHide)

		dropdown:ClearAllPoints()
		dropdown:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		dropdown:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 15, 0)
		dropdown:SetScript("OnHide", nil)

		local left = _G[dropdown:GetName() .. "Left"]
		local middle = _G[dropdown:GetName() .. "Middle"]
		local right = _G[dropdown:GetName() .. "Right"]

		left:Hide()
		right:Hide()

		self.selector = middle

		middle:ClearAllPoints()
		right:ClearAllPoints()

		middle:SetPoint("LEFT", frame, "LEFT", 7, 0)
		middle:SetPoint("RIGHT", frame, "RIGHT", -7, 0)
		middle:SetPoint("TOP", dropdown, "TOP", 0, 0)
		middle:SetPoint("BOTTOM", dropdown, "BOTTOM", 0, 0)

		right:SetPoint("TOPRIGHT", dropdown, "TOPRIGHT", -10, 19) -- set up button placement here

		local button = _G[dropdown:GetName() .. "Button"]
		self.button = button
		button.obj = self
		button:SetScript("OnEnter", Control_OnEnter)
		button:SetScript("OnLeave", Control_OnLeave)
		button:SetScript("OnClick", Dropdown_TogglePullout)

		local button_cover = CreateFrame("BUTTON", nil, self.frame)
		self.button_cover = button_cover
		button_cover.obj = self
		button_cover:SetPoint("TOPLEFT", self.frame, "BOTTOMLEFT", 0, 25)
		button_cover:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMRIGHT")
		button_cover:SetScript("OnEnter", Control_OnEnter)
		button_cover:SetScript("OnLeave", Control_OnLeave)
		button_cover:SetScript("OnClick", Dropdown_TogglePullout)

		self.button.normalTexture    = _G[dropdown:GetName().."ButtonNormalTexture"]
		self.button.disabledTexture  = _G[dropdown:GetName().."ButtonDisabledTexture"]
		self.button.pushedTexture    = _G[dropdown:GetName().."ButtonPushedTexture"]
		self.button.highlightTexture = _G[dropdown:GetName().."ButtonHighlightTexture"]
		
		local text = _G[dropdown:GetName() .. "Text"]
		self.text = text
		text.obj = self
		text:ClearAllPoints()
		text:SetPoint("RIGHT", right, "RIGHT" , -43, 2)
		text:SetPoint("LEFT", left, "LEFT", 25, 2)
		text:SetShadowColor(0, 0, 0, 0)
		
		local label = frame:CreateFontString(nil, "BACKGROUND") -- OVERLAY
		label:SetPoint("TOPLEFT",frame,"TOPLEFT",0,0)
		label:SetPoint("TOPRIGHT",frame,"TOPRIGHT",0,0)
		label:SetJustifyH("LEFT")
		label:SetJustifyV("MIDDLE")
		label:SetHeight(18)
		label:Hide()
		self.label = label

		Gui:RegisterAsWidget(self)
		return self
	end

	Gui:RegisterWidgetType(widgetType, Constructor, widgetVersion)
end