
local Healcsi      = Healcsi
local Console      = Healcsi.Console
local Media        = Healcsi.Media
local Gui          = Healcsi.Gui
local L            = Healcsi.Locale
local Modules      = Healcsi.Modules
local UnitAuras    = Healcsi.Modules.UnitAuras
local Editor       = Healcsi.Editor

Healcsi:RegisterEventCallback("UpdateEditorLayout", function()
	UnitAuras:RefreshScrollBar()
end)

Healcsi:RegisterEventCallback("GlobalConfigRefresh", function()
	UnitAuras.parentFrame:ReleaseChildren()
	UnitAuras:CreateUnitAurasEditorView(UnitAuras.parentFrame)
end)

function UnitAuras:RefreshScrollBar()
	if (not(self.scrollFrame)) then return end

	local pos = self.scrollFrame.scrollbar:GetValue()

	self.scrollFrame:DoLayout()
	self.scrollFrame:FixScroll() -- avoid jumpy scrollbar

	self.scrollFrame.scrollbar:SetValue(pos) -- set position
	self.scrollFrame:SetScroll(pos) -- just to be sure. don't think this is needed btw...
end

function UnitAuras:DrawBuffGroup(container)
	local db = Healcsi.db.profile.UnitAuras.buff
	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Buffs"])

	Editor:CreateAurasEditor(holder, db, "Buffs", "UnitAuraConfigChanged")
end

function UnitAuras:DrawExtraBuffGroup(container)
	local db = Healcsi.db.profile.UnitAuras.buff.check
	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["ExtraBuffs"])

	Editor:CreateExtraBuffsEditor(holder, db, "ExtraBuffs", "UnitAuraConfigChanged")
end

function UnitAuras:DrawDebuffGroup(container)
	local db = Healcsi.db.profile.UnitAuras.debuff
	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Debuffs"])

	Editor:CreateAurasEditor(holder, db, "Debuffs", "UnitAuraConfigChanged")
end

function UnitAuras:DrawDispelGroup(container)
	local db = Healcsi.db.profile.UnitAuras.dispel
	local cont = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(cont, L["Dispels"])

	local padding = Editor:CreateGroup(cont, true, nil, {10, 0, 0, 0})
	local holder = Editor:CreateGroup(padding, true, nil, {0, 10, 10, 10}, {5, 0, 5, 0})

	Editor:CreateCheckBox(holder, false, 300, L["ShowDispels"], db, "show", "UnitAurasShowDispels", "UnitAuraConfigChanged")

	Editor:CreateHorizLine(holder)

	Editor:CreateSlider(holder, false, 300, L["Width"], 0, 14, 1, false, db, "width", "UnitAurasDispelWidth", "UnitAuraConfigChanged")
	Editor:CreateSlider(holder, false, 300, L["Alpha"], 0, 1, 0.01, true, db.flash, "alpha", "UnitAurasDispelAlpha", "UnitAuraConfigChanged")

	Editor:CreateCheckBox(holder, false, 300, L["Flash"], db.flash, "enable", "UnitAurasDispelFlash", "UnitAuraConfigChanged")
end

function UnitAuras:DrawDebuffHLGroup(container)
	local db = Healcsi.db.profile.UnitAuras.debuffHighlight
	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)

	Editor:CreateDebuffHighlightEditor(holder, db, "DebuffHighlight", "DebuffHighlightConfigChanged", UnitAuras)
end

local function SelectGroup(container, event, group)
	container:ReleaseChildren()

	local scrollFrame = Gui:Create("Healcsi-ScrollFrame")
	scrollFrame:SetLayout("Flow")
	scrollFrame:SetFullWidth(true)
	scrollFrame:SetFullHeight(true)
	UnitAuras.scrollFrame = scrollFrame
	container:AddChild(scrollFrame)

	if (group == "buffs") then
		UnitAuras:DrawBuffGroup(scrollFrame)
	elseif (group == "extrabuffs") then
		UnitAuras:DrawExtraBuffGroup(scrollFrame)
	elseif (group == "debuffs") then
		UnitAuras:DrawDebuffGroup(scrollFrame)
	elseif (group == "dispels") then
		UnitAuras:DrawDispelGroup(scrollFrame)
	elseif (group == "debuffHL") then
		UnitAuras:DrawDebuffHLGroup(scrollFrame)
	end

	Healcsi.db.profile.UnitAuras.selectedTab = group

	UnitAuras:RefreshScrollBar()
end

function UnitAuras:CreateUnitAurasEditorView(frame)

	local tab =  Gui:Create("Healcsi-TabGroup")
	tab:SetFullWidth(true)
	tab:SetFullHeight(true)
	tab:SetLayout("Flow") -- Do not set Fill or else it will freeze... brrrr...

	local tabs = {
		{text="Buffs",            value="buffs"},
		{text="Extra Buffs",      value="extrabuffs"},
		{text="Debuffs",          value="debuffs"},
		{text="Dispels",          value="dispels"},
		{text="Debuff Highlight", value="debuffHL"}
	}

	tab:SetTabs(tabs)
	tab:SetCallback("OnGroupSelected", SelectGroup)

	local selectedTab = Healcsi.db.profile.UnitAuras.selectedTab or "buffs"

	local found = false
	for i, currTab in ipairs(tabs) do
		if(currTab["value"] == selectedTab) then found = true end
	end

	if (found) then
		tab:SelectTab(selectedTab)
	end
	
	frame:AddChild(tab)

end

function UnitAuras:CreateEditor()

	--[[
	local scrollFrame = Gui:Create("Healcsi-ScrollFrame")
	scrollFrame:SetLayout("Flow")
	scrollFrame:SetFullWidth(true)
	scrollFrame:SetFullHeight(true)
	self.scrollFrame = scrollFrame
	]]

	local parentFrame = Gui:Create("SimpleGroup")
	parentFrame:SetLayout("Flow")
	parentFrame:SetFullWidth(true)
	parentFrame:SetFullHeight(true)
	self.parentFrame = parentFrame

	-- Put page content here

	self:CreateUnitAurasEditorView(parentFrame)

	-- End of page content

	--scrollFrame:AddChild(parentFrame)
	--return scrollFrame

	return parentFrame
end