
local Healcsi       = Healcsi
local Console       = Healcsi.Console
local Media         = Healcsi.Media
local Gui           = Healcsi.Gui
local L             = Healcsi.Locale
local Modules       = Healcsi.Modules
local Editor        = Healcsi.Editor
local UnitAuras     = Healcsi.Modules.UnitAuras
local UnitButtons   = Healcsi.Modules.UnitButtons

function UnitAuras:Initialize()

	self:InitDebuffHighLight()

	self.initDone = true
end

function UnitAuras:IsInitDone()
	return self.initDone
end

-- ==============================================================
-- Create

function UnitAuras:CreateAuraFrame(key, name, parent, list, db)
	local frame = CreateFrame("Frame", name, parent, "CompactAuraTemplate")
	frame:SetSize(db.size, db.size)
	frame.key = key


	if (key == "debuff") then
		local border = frame:CreateTexture(name.."Border", "OVERLAY")
		frame.border = border
		--border:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")
		--border:SetTexCoord(0.296875, 0.5703125, 0, 0.515625)
		border:SetTexture([[Interface\Addons\Healcsi\Media\UI\select\select-1]])
		border:SetAllPoints(frame)
	end

	frame:SetFrameLevel(50)
	frame.cooldown:SetFrameLevel(100)
	frame.cooldown:SetHideCountdownNumbers(false)
	frame.cooldown:SetSwipeColor(1, 1, 1, 0.6)

	local stackCount = CreateFrame("Frame", name.."StackCount", frame)
	stackCount:SetAllPoints(frame)
	stackCount:SetFrameLevel(150)

	local text = stackCount:CreateFontString(name.."StackCountText", "OVERLAY")
	text:SetPoint(db.stack.anchor, frame, db.stack.anchor, db.stack.offset.x, db.stack.offset.y)
	if (db.stack.hasOutline) then
		text:SetFont(Healcsi:GetMedia("font", db.stack.font), db.stack.size, "OUTLINE")
	else
		text:SetFont(Healcsi:GetMedia("font", db.stack.font), db.stack.size)
	end

	frame.stacks = text
	frame.count:Hide()

	local regions = { frame.cooldown:GetRegions() }
	local f
	for f=1, #regions do -- I don't know how to do otherwise... but I'm sure there is an easier way...
		if (regions[f].SetFont) then
			regions[f]:ClearAllPoints()
			regions[f]:SetPoint(db.remaining.anchor, frame.cooldown, db.remaining.anchor, db.remaining.offset.x, db.remaining.offset.y)
			if (db.remaining.hasOutline) then
				regions[f]:SetFont(Healcsi:GetMedia("font", db.remaining.font), db.remaining.size, "OUTLINE")
			else
				regions[f]:SetFont(Healcsi:GetMedia("font", db.remaining.font), db.remaining.size)
			end
		end
	end

	if (db.demo) then
		frame.icon:SetTexture("Interface\\Icons\\Ability_Garrison_OrangeBird")

		local demoStacks = math.random(1, 15)
		if (demoStacks > 2) then
			frame.stacks:SetText(demoStacks-2)
		else
			frame.stacks:SetText("")
		end

		local duration = math.random(20, 60)
		local expires = GetTime() + duration

		frame.cooldown:SetCooldown(expires - duration, duration)
		frame.cooldown:Show()

		if (key == "debuff") then
			local debuffTypes = {"Curse", "Disease", "Magic", "Poison"}
			local color = DebuffTypeColor[debuffTypes[math.random(#debuffTypes)]]
			frame.border:SetVertexColor(color.r, color.g, color.b)
		end
	else
		frame:Hide()
	end
	
	frame.icon:SetTexCoord(db.zoom, 1-db.zoom, db.zoom, 1-db.zoom)

	table.insert(list, frame)
	return frame

end

-- ==============================================================
-- Update

function UnitAuras:UpdateAllFrameLayout()
	for _, frame in ipairs(UnitButtons.unitFrames) do
		self:UpdateFrameLayout(frame)
	end
end

function UnitAuras:UpdateFrameLayout(frame)

	local db = Healcsi.db.profile.UnitButtons.size

	local fullWidth  = db.unitWidth - 2
	local fullheight = db.unitHeight - 2 - db.powerHeight

	local i, parent, fitWidth, fitHeight

	local lastHeadIndex, line

	-- ==============================================================
	db = Healcsi.db.profile.UnitAuras.buff
	parent   = frame.buffParent

	if (frame.buffFrames) then
		for i = 1, #(frame.buffFrames) do
			frame.buffFrames[i]:Hide()
		end
	end
	frame.buffFrames = {}

	fitWidth, fitHeight = fullWidth * db.availableWidth, fullheight * db.availableHeight

	parent:SetAlpha(db.alpha)
	parent:ClearAllPoints()
	parent:SetPoint(db.anchor, frame.healthBar, db.anchor, db.offset.x, db.offset.y)
	parent:SetSize(fitWidth, fitHeight)
	if (db.demo) then
		if (parent.demoTexture) then
			parent.demoTexture:SetVertexColor(Healcsi:GetColor("Green2", 0.3))
		else
			local demoTexture = parent:CreateTexture(nil, "BACKGROUND")
			demoTexture:SetAllPoints(parent)
			Healcsi:DecorateTexture(demoTexture, "background", "Solid", "Green2", 0.3)
			parent.demoTexture = demoTexture
		end
	elseif (parent.demoTexture) then
		parent.demoTexture:SetVertexColor(0, 0, 0, 0)
	end

	lastHeadIndex = 1
	line = 1

	for i = 1, db.count do
		local buff = self:CreateAuraFrame("buff", frame:GetName().."Buff"..i, parent, frame.buffFrames, db)
		if (i == 1) then
			buff:SetPoint(db.anchor, frame.healthBar, db.anchor, db.offset.x, db.offset.y)
		else
			if (db.grow == "LEFT") then
				local k = i - (lastHeadIndex-1)
				if ( (k*db.size) < fitWidth ) then
					buff:SetPoint("RIGHT", frame.buffFrames[i - 1], "LEFT")
					if ((line*db.size) > fitHeight) then
						buff:Hide()
					end
				else
					if (db.anchor == "BOTTOMLEFT" or db.anchor == "BOTTOMRIGT") then
						buff:SetPoint("BOTTOM", frame.buffFrames[lastHeadIndex], "TOP")
					else
						buff:SetPoint("TOP", frame.buffFrames[lastHeadIndex], "BOTTOM")
					end
					lastHeadIndex = i
					line = line + 1
					if ((line*db.size) > fitHeight) then
						buff:Hide()
					end
				end
			elseif (db.grow == "RIGHT") then
				local k = i - (lastHeadIndex-1)
				if ( (k*db.size) < fitWidth ) then
					buff:SetPoint("LEFT", frame.buffFrames[i - 1], "RIGHT")
					if ((line*db.size) > fitHeight) then
						buff:Hide()
					end
				else
					if (db.anchor == "BOTTOMLEFT" or db.anchor == "BOTTOMRIGT") then
						buff:SetPoint("BOTTOM", frame.buffFrames[lastHeadIndex], "TOP")
					else
						buff:SetPoint("TOP", frame.buffFrames[lastHeadIndex], "BOTTOM")
					end
					lastHeadIndex = i
					line = line + 1
					if ((line*db.size) > fitHeight) then
						buff:Hide()
					end
				end
			elseif (db.grow == "UP") then
				buff:SetPoint("BOTTOM", frame.buffFrames[i - 1], "TOP")
			elseif (db.grow == "DOWN") then
				buff:SetPoint("TOP", frame.buffFrames[i - 1], "BOTTOM")
			end
		end
	end


	-- ==============================================================

	-- ==============================================================
	db = Healcsi.db.profile.UnitAuras.debuff
	parent   = frame.debuffParent

	if (frame.debuffFrames) then
		for i = 1, #(frame.debuffFrames) do
			frame.debuffFrames[i]:Hide()
		end
	end
	frame.debuffFrames = {}

	fitWidth, fitHeight = fullWidth * db.availableWidth, fullheight * db.availableHeight

	parent:SetAlpha(db.alpha)
	parent:ClearAllPoints()
	parent:SetPoint(db.anchor, frame.healthBar, db.anchor, db.offset.x, db.offset.y)
	parent:SetSize(fitWidth, fitHeight)
	if (db.demo) then
		if (parent.demoTexture) then
			parent.demoTexture:SetVertexColor(Healcsi:GetColor("Red", 0.3))
		else
			local demoTexture = parent:CreateTexture(nil, "BACKGROUND")
			demoTexture:SetAllPoints(parent)
			Healcsi:DecorateTexture(demoTexture, "background", "Solid", "Red", 0.3)
			parent.demoTexture = demoTexture
		end
	elseif (parent.demoTexture) then
		parent.demoTexture:SetVertexColor(0, 0, 0, 0)
	end

	lastHeadIndex = 1
	line = 1

	for i = 1, db.count do
		local debuff = self:CreateAuraFrame("debuff", frame:GetName().."Debuff"..i, parent, frame.debuffFrames, db)
		if (i == 1) then
			debuff:SetPoint(db.anchor, frame.healthBar, db.anchor, db.offset.x, db.offset.y)
		else
			if (db.grow == "LEFT") then
				local k = i - (lastHeadIndex-1)
				if ( (k*db.size) < fitWidth ) then
					debuff:SetPoint("RIGHT", frame.debuffFrames[i - 1], "LEFT")
					if ((line*db.size) > fitHeight) then
						debuff:Hide()
					end
				else
					if (db.anchor == "BOTTOMLEFT" or db.anchor == "BOTTOMRIGT") then
						debuff:SetPoint("BOTTOM", frame.debuffFrames[lastHeadIndex], "TOP")
					else
						debuff:SetPoint("TOP", frame.debuffFrames[lastHeadIndex], "BOTTOM")
					end
					lastHeadIndex = i
					line = line + 1
					if ((line*db.size) > fitHeight) then
						debuff:Hide()
					end
				end
			elseif (db.grow == "RIGHT") then
				local k = i - (lastHeadIndex-1)
				if ( (k*db.size) < fitWidth ) then
					debuff:SetPoint("LEFT", frame.debuffFrames[i - 1], "RIGHT")
					if ((line*db.size) > fitHeight) then
						debuff:Hide()
					end
				else
					if (db.anchor == "BOTTOMLEFT" or db.anchor == "BOTTOMRIGT") then
						debuff:SetPoint("BOTTOM", frame.debuffFrames[lastHeadIndex], "TOP")
					else
						debuff:SetPoint("TOP", frame.debuffFrames[lastHeadIndex], "BOTTOM")
					end
					lastHeadIndex = i
					line = line + 1
					if ((line*db.size) > fitHeight) then
						debuff:Hide()
					end
				end
			elseif (db.grow == "UP") then
				debuff:SetPoint("BOTTOM", frame.debuffFrames[i - 1], "TOP")
			elseif (db.grow == "DOWN") then
				debuff:SetPoint("TOP", frame.debuffFrames[i - 1], "BOTTOM")
			end
		end
	end

	-- ==============================================================

end

local function UpdateAuras(self, name, icon, count, duration, expires, dispelType)
	if name then
		self.icon:SetTexture(icon)

		if count and count > 1 then
			self.stacks:SetText(count)
			self.stacks:Show()
		else
			self.stacks:Hide()
		end

		if expires and expires ~= 0 then
			self.cooldown:SetCooldown(expires - duration, duration)
			self.cooldown:Show()
		else
			self.cooldown:Hide()
		end

		if dispelType then
			local color = DebuffTypeColor[dispelType] or DebuffTypeColor["none"]
			self.border:SetVertexColor(color.r, color.g, color.b) -- Curse, Disease, Magic, Poison
		end

		self:Show()
	else
		self:Hide()
	end
end

function UnitAuras:BuffHelper(frame, buffId)
	local displayedUnit = frame.displayedUnit
	if (not(displayedUnit)) then return end

	return UnitBuff(displayedUnit, GetSpellInfo(buffId), nil, "PLAYER")
end


function UnitAuras:UpdateBuffs(frame)
	local db = Healcsi.db.profile.UnitAuras.buff

	if (not(db.show)) then
		frame.buffParent:Hide()
		return
	end

	local displayedUnit = frame.displayedUnit
	if (not(displayedUnit)) then return end

	local parent = frame.buffParent
	parent:Show()

	if (db.demo) then return end

	--[[
	Moved to config where you can edit them!
	local checkTheseBuffsBefore = {
		["Priest-Atonement"]          = 81749,
		["Priest-Echo-of-Light"]      = 77489,
		["Priest-Prayer-of-Mending"]  = 33076,
		["Shaman-Queen-s-Decree"]     = 207360
	}

	local checkTheseBuffsAfter = {
		["Shman-Ancestral-Vigor"] = 207400
	}
	]]

	local checkTheseBuffsBefore = {}
	local checkTheseBuffsAfter  = {}
	local playerClass = select(2, UnitClass("player"))

	for key, value in ipairs(db.check.before) do
		if (playerClass == value.class or "ALL" == value.class) then
			checkTheseBuffsBefore[value.class.."-"..key] = value.spellId
		end
	end

	for key, value in ipairs(db.check.after) do
		if (playerClass == value.class or "ALL" == value.class) then
			checkTheseBuffsAfter[value.class.."-"..key] = value.spellId
		end
	end

	local appliedList = {}

	local b = 1
	for key, value in pairs(checkTheseBuffsBefore) do 
		if(self:BuffHelper(frame, value)) then
			if (b > db.count) then break end
			local name, rank, icon, count, dispelType, duration, expires, caster = self:BuffHelper(frame, value)
			if (caster == "player" and not(appliedList[name])) then
				appliedList[name] = 1
				UpdateAuras(frame.buffFrames[b], name, icon, count, duration, expires)
				b = b + 1
			end
		end
	end

	local beforeBuffCount = b
	local i
	for i = b, db.count do
		local name, rank, icon, count, dispelType, duration, expires = UnitBuff(displayedUnit, i-(beforeBuffCount-1), "RAID|PLAYER")
		if (name) then b = b + 1 end
		UpdateAuras(frame.buffFrames[i], name, icon, count, duration, expires)
	end

	if (b <= db.count) then
		for key, value in pairs(checkTheseBuffsAfter) do 
			if(self:BuffHelper(frame, value)) then
				if (b > db.count) then break end
				local name, rank, icon, count, dispelType, duration, expires, caster = self:BuffHelper(frame, value)
				if (caster == "player" and not(appliedList[name])) then
					appliedList[name] = 1
					UpdateAuras(frame.buffFrames[b], name, icon, count, duration, expires)
					b = b + 1
				end
			end
		end
	end

end

function UnitAuras:UpdateDebuffs(frame)
	local db = Healcsi.db.profile.UnitAuras.debuff

	if (not(db.show)) then
		frame.debuffParent:Hide()
		return
	end

	local displayedUnit = frame.displayedUnit
	if (not(displayedUnit)) then return end

	local parent = frame.debuffParent
	parent:Show()

	if (db.demo) then
		return
	end

	local filter = ""
	if (db.onlyDispellable) then filter = "RAID" end

	local index = 0
	local i
	for i = 1, 40 do
		if index >= db.count then
			break
		end

		local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, nameplateShowPersonal, spellID, canApplyAura, isBossDebuff, _, nameplateShowAll, timeMod, value1, value2, value3 = UnitDebuff(displayedUnit, i, filter)

		if not name then
			break
		end

		if ((duration) and (duration < 600)) then
			index = index + 1
			UpdateAuras(frame.debuffFrames[index], name, icon, count, duration, expires, dispelType or "none")
		end

	end

	for i = index + 1, db.count do
		frame.debuffFrames[i]:Hide()
	end
	
end

function UnitAuras:UpdateDispels(frame)
	local db = Healcsi.db.profile.UnitAuras.dispel

	if (not(db.show)) then
		frame.dispelParent:Hide()
		return
	end

	local displayedUnit = frame.displayedUnit
	if (not(displayedUnit)) then return end

	local parent = frame.dispelParent

	parent:ClearAllPoints()
	parent:SetPoint("TOPLEFT",     frame.healthBar, "TOPLEFT", 0, 0)
	parent:SetPoint("BOTTOMRIGHT", frame.healthBar, "BOTTOMLEFT", db.width, 0)

	if (not(parent.texture)) then
		parent.texture = parent:CreateTexture(nil, "OVERLAY")
		parent.texture:SetAllPoints(parent)
		parent.texture:SetTexture(Healcsi:GetMedia("background", "Solid"))
		parent.texture:SetVertexColor(1, 0, 0)
	end

	local foundOne = false
	local i
	for i = 1, 5 do
		if (not(foundOne)) then
			local name, rank, icon, count, dispelType = UnitDebuff(displayedUnit, i, "RAID")
			if ((name) and (dispelType)) then
				parent.texture.color = DebuffTypeColor[dispelType] or DebuffTypeColor["none"]
				parent:Show()
				foundOne = true
			else
				parent.texture.color = DebuffTypeColor["none"]
				parent:Hide()
			end
		end
	end

	if (not(parent.flasher)) then
		parent.flasher = CreateFrame("Frame")
		parent.flasher.obj = parent
		parent.flasher:SetScript("OnUpdate", function(self, elapsed)
			self.timer = (self.timer or 0) + elapsed
			if (self.timer > (db.flash.speed/100)) then
				self.timer = 0
				-- Tick:

				self.alpha = self.alpha or 1
				self.direction = self.direction or "hide"
				if (self.direction == "hide") then
					self.alpha = self.alpha - 0.05
					if (self.alpha < 0) then
						self.alpha = 0
						self.direction = "show"
					end
				else
					self.alpha = self.alpha + 0.05
					if (self.alpha > db.flash.alpha) then
						self.alpha = db.flash.alpha
						self.direction = "hide"
					end
				end

				if (db.flash.enable) then
					self.obj.texture:SetVertexColor(self.obj.texture.color.r, self.obj.texture.color.g, self.obj.texture.color.b, self.alpha)
				else
					self.obj.texture:SetVertexColor(self.obj.texture.color.r, self.obj.texture.color.g, self.obj.texture.color.b, db.flash.alpha)
				end
			end
		end)
	end

end

function UnitAuras:UpdateAuras(frame)
	local displayedUnit = frame.displayedUnit
	if (not(displayedUnit)) then return end

	if (not(frame.buffFrames) or not(frame.debuffFrames)) then
		UnitAuras:UpdateFrameLayout(frame) -- we should allow this to be done even in combat
	end

	self:UpdateBuffs(frame)
	self:UpdateDebuffs(frame)
	self:UpdateDispels(frame)
end

function UnitAuras:UpdateAllAuras()
	for _, frame in ipairs(UnitButtons.unitFrames) do
		UnitAuras:UpdateAuras(frame)
	end
end


-- ==============================================================
-- Notify

Healcsi:RegisterEventCallback("OnUnitNotify", function(frame, event, ...)
	if (event == "OnAurasChange") then
		UnitAuras:UpdateAuras(frame)
	elseif (event == "OnUnitSizeChange") then
		UnitAuras:UpdateFrameLayout(frame)
	end
end)

Healcsi:RegisterEventCallback("UnitAuraConfigChanged", function()
	-- Healcsi:Print("UnitAuraConfigChanged")
	UnitAuras:UpdateAllFrameLayout()
	UnitAuras:UpdateAllAuras()
end)
