
local Healcsi      = Healcsi
local Console      = Healcsi.Console
local Media        = Healcsi.Media
local Gui          = Healcsi.Gui
local L            = Healcsi.Locale
local Modules      = Healcsi.Modules
local UnitButtons  = Healcsi.Modules.UnitButtons
local Editor       = Healcsi.Editor

Healcsi:RegisterEventCallback("UpdateEditorLayout", function()
	UnitButtons:RefreshScrollBar()
end)

Healcsi:RegisterEventCallback("GlobalConfigRefresh", function()
	if(UnitButtons.parentFrame) then
		UnitButtons.parentFrame:ReleaseChildren()
		UnitButtons:CreateUnitButtonsEditorView(UnitButtons.parentFrame)
	end
end)

function UnitButtons:RefreshScrollBar()
	if (not(self.scrollFrame)) then return end

	local pos = self.scrollFrame.scrollbar:GetValue()

	self.scrollFrame:DoLayout()
	self.scrollFrame:FixScroll() -- avoid jumpy scrollbar

	self.scrollFrame.scrollbar:SetValue(pos) -- set position
	self.scrollFrame:SetScroll(pos) -- just to be sure. don't think this is needed btw...
end


function UnitButtons:DrawGeneralGroup(container)
	local db = Healcsi.db.profile.UnitButtons.size

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Dimensions"])

	local line1 = Editor:CreateInlineGroup(holder, true, nil, {10, 5, 10, 5}, {0, 5, 5, 5})
	Editor:CreateSlider(line1, false, 300, L["UnitWidth"], 10, 200, 1, false, db, "unitWidth", "UnitButtonUnitWidth", "UnitButtonConfigChanged", true)
	Editor:CreateSlider(line1, false, 300, L["UnitHeight"], 10, 200, 1, false, db, "unitHeight", "UnitButtonUnitHeight", "UnitButtonConfigChanged", true)

	local line2 = Editor:CreateInlineGroup(holder, true, nil, {10, 5, 10, 5}, {5, 5, 0, 5})
	Editor:CreateSlider(line2, false, 300, L["Spacing"], 0, 25, 1, false, db, "spacing", "UnitButtonSpacing", "UnitButtonConfigChanged", true)
	Editor:CreateSlider(line2, false, 300, L["PowerHeight"], 0, 25, 1, false, db, "powerHeight", "UnitButtonPowerHeight", "UnitButtonConfigChanged")
	Editor:CreateSlider(line2, false, 300, L["PetPowerHeight"], 0, 25, 1, false, db, "petPowerHeight", "UnitButtonPetPowerHeight", "UnitButtonConfigChanged")

	Editor:CreateHeaderlabel(holder, L["Outrange"])

	local line3 = Editor:CreateInlineGroup(holder, true, nil, {10, 5, 10, 5}, {5, 5, 0, 5})
	Editor:CreateSlider(line3, false, 300, L["OutrangeOpacity"], 0, 1, 0.01, true, db, "outrangeOpacity", "UnitButtonOutrangeOpacity", "UnitButtonConfigChanged", true)

	--[[
	Editor:CreateHorizLine(holder)

	db = Healcsi.db.profile.UnitButtons.icons

	Editor:CreateHeaderlabel(holder, L["Icons"])

	local line4 = Editor:CreateInlineGroup(holder, true, nil, {10, 5, 10, 5}, {5, 5, 0, 5})
	
	Editor:CreateCheckBox(line4, false, 500, L["ShowPrivilegeIcon"],  db.privilege,  "show", "UnitButtonShowPrivilegeIcon",  "UnitButtonConfigChanged")
	Editor:CreateCheckBox(line4, false, 500, L["ShowRoleIcon"],       db.role,       "show", "UnitButtonShowRoleIcon",       "UnitButtonConfigChanged")
	Editor:CreateCheckBox(line4, false, 500, L["ShowRaidTargetIcon"], db.raidTarget, "show", "UnitButtonShowRaidTargetIcon", "UnitButtonConfigChanged")
	]]
end

function UnitButtons:DrawBackgroundGroup(container)
	local db = Healcsi.db.profile.UnitButtons.background

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Background"])

	local line1 = Editor:CreateInlineGroup(holder, true, nil, {10, 5, 10, 5}, {0, 5, 5, 5})
	Editor:CreateColorPicker(line1, false, 300, true, L["BackgroundColor"], db, "color", "UnitButtonBgColor", "UnitButtonConfigChanged")
	Editor:CreateBackground(line1, false, 300, L["BackgroundTex"], db, "file", "UnitButtonBgFile", "UnitButtonConfigChanged")
end

function UnitButtons:DrawHealthGroup(container)
	local db = Healcsi.db.profile.UnitButtons.health

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Background"])

	local line1 = Editor:CreateGroup(holder, true, nil, {10, 0, 10, 10})

	Editor:CreateColorPicker(line1, false, 300, true, L["OfflineFixColor"],  db.color.offline,  "fix", "UnitButtonOfflineFixColor", "UnitButtonConfigChanged")
	Editor:CreateColorPicker(line1, false, 300, true, L["HostileFixColor"],  db.color.hostile,  "fix", "UnitButtonHostileFixColor", "UnitButtonConfigChanged")
	Editor:CreateColorPicker(line1, false, 300, true, L["FriendlyFixColor"], db.color.friendly, "fix", "UnitButtonFriendlyFixColor", "UnitButtonConfigChanged")
	Editor:CreateColorPicker(line1, false, 300, true, L["NeutralFixColor"],  db.color.neutral,  "fix", "UnitButtonNeutralFixColor", "UnitButtonConfigChanged")
	--Editor:CreateColorPicker(line1, false, 300, true, L["DeadFixColor"],  db.color.dead,        "fix", "UnitButtonDeadFixColor", "UnitButtonConfigChanged")

	Editor:CreateHorizLine(holder)

	Editor:CreateUnitHealthDecorationEditor(holder, db, "Health", "UnitButtonConfigChanged")

	local line2 = Editor:CreateGroup(holder, true, nil, {15, 0, 5, 0})
	Editor:CreateHeaderlabel(line2, L["Underlay"])

	Editor:CreateLabel(holder, true, 300, L["UnderlayDesc"])

	Editor:CreateUnitHealthDecorationEditor(holder, db.underlay, "HealthUnderlay", "UnitButtonConfigChanged")
end

function UnitButtons:DrawPowerGroup(container)
	local db = Healcsi.db.profile.UnitButtons.power

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)

	Editor:CreateHeaderlabel(holder, L["Power"])
	local line1 = Editor:CreateInlineGroup(holder, true, nil, {10, 0, 5, 0})

	Editor:CreateSlider(line1, false, 300, L["PowerHeight"], 0, 25, 1, false, Healcsi.db.profile.UnitButtons.size, "powerHeight", "UnitButtonPowerHeight", "UnitButtonConfigChanged")
	Editor:CreateSlider(line1, false, 300, L["PetPowerHeight"], 0, 25, 1, false, Healcsi.db.profile.UnitButtons.size, "petPowerHeight", "UnitButtonPetPowerHeight", "UnitButtonConfigChanged")

	Editor:CreateUnitPowerDecorationEditor(holder, db, "Power", "UnitButtonConfigChanged")

	local line2 = Editor:CreateGroup(holder, true, nil, {15, 0, 5, 0})
	Editor:CreateHeaderlabel(line2, L["Underlay"])
	Editor:CreateLabel(holder, true, 300, L["PowerUnderlayDesc"])

	Editor:CreateUnitPowerDecorationEditor(holder, db.underlay, "PowerUnderlay", "UnitButtonConfigChanged")
end

function UnitButtons:DrawShieldGroup(container)
	local db = Healcsi.db.profile.UnitButtons.shield

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["DamageAbsorb"])

	Editor:CreateUnitDamageAbsorbDecorationEditor(holder, db, "Shield", "UnitButtonConfigChanged")

end

function UnitButtons:DrawTooltipGroup(container)
	local db = Healcsi.db.profile.UnitButtons.tooltip

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Tooltip"])

	local line = Editor:CreateInlineGroup(holder, true, nil, {20, 0, 5, 0})

	local showModes = {
		["always"]      = "Always",
		["outOfCombat"] = "Out of combat",
		["never"]       = "Never",
	}
	Editor:CreateDropDown(line, false, 300, L["TooltipShowMode"], showModes, db, "show", "UnitButtonShowTooltip", "UnitButtonConfigChanged")

	local posModes = {
		["default"]    = "Default",
		["unitButton"] = "Unit Button"
	}
	Editor:CreateDropDown(line, false, 300, L["TooltipPosMode"], posModes, db, "position", "UnitButtonPositionTooltip", "UnitButtonConfigChanged")

end

function UnitButtons:DrawNameTextGroup(container)
	local db = Healcsi.db.profile.UnitButtons.nameText

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["NameText"])

	local line = Editor:CreateInlineGroup(holder, true, nil, {10, 0, 10, 10})
	Editor:CreateSlider(line, true, 300, L["LengthLimit"], 0.5, 1, 0.01, true, db, "lengthLimit", "UnitButtonNameTextLengthLimit", broadcast)

	Editor:CreateHorizLine(holder)

	Editor:CreateFontEditor(holder, db, "NameText", "UnitButtonConfigChanged")
end

function UnitButtons:DrawHealthTextGroup(container)
	local db = Healcsi.db.profile.UnitButtons.healthText

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["HealthText"])

	Editor:CreateHealthTextEditor(holder, db, "HealthText", "UnitButtonConfigChanged")
end

function UnitButtons:DrawStatusTextGroup(container)
	local db = Healcsi.db.profile.UnitButtons.statusText

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["StatusText"])

	Editor:CreateFontEditor(holder, db, "StatusText", "UnitButtonConfigChanged")
end

function UnitButtons:DrawThreatGroup(container)
	local db = Healcsi.db.profile.UnitButtons.threat

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Threat"])

	Editor:CreateThreatEditor(holder, db, "Threat", "UnitButtonConfigChanged")
end

function UnitButtons:DrawSelectionGroup(container)
	local db = Healcsi.db.profile.UnitButtons.selection

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Selection"])

	Editor:CreateSelectionEditor(holder, db, "Selection", "UnitButtonConfigChanged")
end

function UnitButtons:DrawIconsGroup(container)
	local db = Healcsi.db.profile.UnitButtons.icons

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["Icons"])

	Editor:CreateIconsEditor(holder, db, "Icons", "UnitButtonConfigChanged")
end

function UnitButtons:DrawHealAbsorbGroup(container)
	local db = Healcsi.db.profile.UnitButtons.absorb

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)
	Editor:CreateHeaderlabel(holder, L["HealAbsorb"])

	Editor:CreateHealAbsorbEditor(holder, db, "HealAbsorb", "UnitButtonConfigChanged")
end

function UnitButtons:DrawIncomingGroup(container)
	local db = Healcsi.db.profile.UnitButtons.incHeal

	local holder = Editor:CreateGroup(container, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)

	Editor:CreateHeaderlabel(holder, L["IncomingHeal"])
	Editor:CreateIncHealEditor(holder, db, "IncomingHeal", "UnitButtonConfigChanged")

	Editor:CreateHeaderlabel(holder, L["IncHealColor2Absorb"])
	Editor:CreateHealAbsorbEditor(holder, db.absorb, "IncomingHeal2Absorb", "UnitButtonConfigChanged")
end

local function SelectGroup(container, event, group)
	container:ReleaseChildren()

	local scrollFrame = Gui:Create("Healcsi-ScrollFrame")
	scrollFrame:SetLayout("Flow")
	scrollFrame:SetFullWidth(true)
	scrollFrame:SetFullHeight(true)
	UnitButtons.scrollFrame = scrollFrame
	container:AddChild(scrollFrame)

	if (group == "general") then
		UnitButtons:DrawGeneralGroup(scrollFrame)
	elseif (group == "background") then
		UnitButtons:DrawBackgroundGroup(scrollFrame)
	elseif (group == "health") then
		UnitButtons:DrawHealthGroup(scrollFrame)
	elseif (group == "power") then
		UnitButtons:DrawPowerGroup(scrollFrame)
	elseif (group == "dmgabsorb") then
		UnitButtons:DrawShieldGroup(scrollFrame)
	elseif (group == "incomingheal") then
		UnitButtons:DrawIncomingGroup(scrollFrame)
	elseif (group == "healabsorb") then
		UnitButtons:DrawHealAbsorbGroup(scrollFrame)
	elseif (group == "nametext") then
		UnitButtons:DrawNameTextGroup(scrollFrame)
	elseif (group == "healthtext") then
		UnitButtons:DrawHealthTextGroup(scrollFrame)
	elseif (group == "statustext") then
		UnitButtons:DrawStatusTextGroup(scrollFrame)
	elseif (group == "threat") then
		UnitButtons:DrawThreatGroup(scrollFrame)
	elseif (group == "selection") then
		UnitButtons:DrawSelectionGroup(scrollFrame)
	elseif (group == "tooltip") then
		UnitButtons:DrawTooltipGroup(scrollFrame)
	elseif (group == "icons") then
		UnitButtons:DrawIconsGroup(scrollFrame)
	end

	Healcsi.db.profile.UnitButtons.selectedTab = group

	UnitButtons:RefreshScrollBar()
end


function UnitButtons:CreateUnitButtonsEditorView(frame)

	local tab =  Gui:Create("Healcsi-TabGroup")
	tab:SetFullWidth(true)
	tab:SetFullHeight(true)
	tab:SetLayout("Flow") -- Do not set Fill or else it will freeze... brrrr...

	local tabs = {
		{text="General",       value="general"},
		{text="Background",    value="background"},
		{text="Health",        value="health"},
		{text="Power",         value="power"},
		{text="Damage Absorb", value="dmgabsorb"},
		{text="Incoming Heal", value="incomingheal"},
		{text="Heal Absorb",   value="healabsorb"},
		{text="Name Text",     value="nametext"},
		{text="Health Text",   value="healthtext"},
		{text="Status Text",   value="statustext"},
		{text="Threat",        value="threat"},
		{text="Selection",     value="selection"},
		{text="Tooltip",       value="tooltip"},
		{text="Icons",         value="icons"}
	}

	tab:SetTabs(tabs)
	tab:SetCallback("OnGroupSelected", SelectGroup)

	local selectedTab = Healcsi.db.profile.UnitButtons.selectedTab or "size"

	local found = false
	for i, currTab in ipairs(tabs) do
		if(currTab["value"] == selectedTab) then found = true end
	end

	if (found) then
		tab:SelectTab(selectedTab)
	end
	
	frame:AddChild(tab)

end

function UnitButtons:CreateEditor()

	--[[
	local scrollFrame = Gui:Create("Healcsi-ScrollFrame")
	scrollFrame:SetLayout("Flow")
	scrollFrame:SetFullWidth(true)
	scrollFrame:SetFullHeight(true)
	self.scrollFrame = scrollFrame
	]]

	local parentFrame = Gui:Create("SimpleGroup")
	parentFrame:SetLayout("Flow")
	parentFrame:SetFullWidth(true)
	parentFrame:SetFullHeight(true)
	self.parentFrame = parentFrame

	-- Put page content here

	self:CreateUnitButtonsEditorView(parentFrame)

	-- End of page content

	--scrollFrame:AddChild(parentFrame)
	--return scrollFrame

	return parentFrame
end