
local Healcsi    = Healcsi
local Console    = Healcsi.Console
local Media      = Healcsi.Media
local Gui        = Healcsi.Gui
local L          = Healcsi.Locale
local Modules    = Healcsi.Modules
local UnitFrames = Healcsi.Modules.UnitFrames
local Editor     = Healcsi.Editor

local _G = _G
local RegisterStateDriver = RegisterStateDriver
local SetCVar = SetCVar

Healcsi:RegisterEventCallback("UpdateEditorLayout", function()
	UnitFrames:RefreshScrollBar()
end)

Healcsi:RegisterEventCallback("GlobalConfigRefresh", function()
	UnitFrames.parentFrame:ReleaseChildren()
	UnitFrames:CreateUnitFramesEditorView(UnitFrames.parentFrame)
end)

Healcsi:RegisterEventCallback("UnitFrameConfigChanged", function()
	UnitFrames.parentFrame:ReleaseChildren()
	UnitFrames:CreateUnitFramesEditorView(UnitFrames.parentFrame)
end)

function UnitFrames:RefreshScrollBar()
	local pos = self.scrollFrame.scrollbar:GetValue()

	self.scrollFrame:DoLayout()
	self.scrollFrame:FixScroll() -- avoid jumpy scrollbar

	self.scrollFrame.scrollbar:SetValue(pos) -- set position
	self.scrollFrame:SetScroll(pos) -- just to be sure. don't think this is needed btw...
end

--"UnitFrameConfigChanged"


function UnitFrames:CreateUnitFramesEditorView(frame)

	local db = Healcsi.db.profile.UnitFrames
	local holder = Editor:CreateGroup(frame, true, nil, {10, 0, 10, 10}, {0, 5, 0, 5}, "BlueSelected", 0.3)

	Editor:CreateHeaderlabel(holder, L["General"])
	local general = Editor:CreateGroup(holder, true, nil, {10, 0, 10, 10}, {0, 0, 0, 0})
	Editor:CreateCheckBox(general, false, 300, L["lockPositionCB"], db.general, "lockPosition", "UnitFramesLockPosition", "UnitFrameConfigChanged")
	Editor:CreateCheckBox(general, false, 300, L["alignHorizontallyCB"], db.general, "alignHorizontally", "UnitFramesAlignHorizontally", "UnitFrameConfigChanged", true)

	Editor:CreateHorizLine(holder)
	Editor:CreateHeaderlabel(holder, L["Solo"])
	local solo = Editor:CreateGroup(holder, true, nil, {10, 0, 10, 10}, {0, 0, 0, 0})
	Editor:CreateCheckBox(solo, false, 300, L["showWhenNotGroupedCB"], db.solo, "showWhenNotGrouped", "UnitFramesShowWhenNotGrouped", "UnitFrameConfigChanged", true)

	Editor:CreateHorizLine(holder)
	Editor:CreateHeaderlabel(holder, L["Party"])
	local party = Editor:CreateGroup(holder, true, nil, {10, 0, 10, 10}, {0, 0, 0, 0})
	Editor:CreateCheckBox(party, false, 300, L["showWhenInPartyCB"], db.party, "showWhenInParty", "UnitFramesShowWhenInParty", "UnitFrameConfigChanged", true)
	Editor:CreateCheckBox(party, false, 300, L["showPartyPetsCB"], db.party, "showPartyPets", "UnitFramesShowPartyPets", "UnitFrameConfigChanged", true)
	Editor:CreateCheckBox(party, false, 300, L["hideBlizzPartyFrames"], db.party, "hideBlizzPartyFrames", "UnitFramesHideBlizzPartyFrames", "UnitFrameConfigChanged", true)

	Editor:CreateHorizLine(holder)
	Editor:CreateHeaderlabel(holder, L["Raid"])
	local raid = Editor:CreateGroup(holder, true, nil, {10, 0, 10, 10}, {0, 0, 0, 0})
	Editor:CreateCheckBox(raid, false, 300, L["showWhenInRaidCB"], db.raid, "showWhenInRaid", "UnitFramesShowWhenInRaid", "UnitFrameConfigChanged", true)
	Editor:CreateCheckBox(raid, false, 300, L["showRaidPetsCB"], db.raid, "showRaidPets", "UnitFramesShowRaidPets", "UnitFrameConfigChanged", true)
	Editor:CreateCheckBox(raid, false, 300, L["keepRaidGroupsTogetherCB"], db.raid, "keepGroupsTogether", "UnitFramesKeepGroupsTogether", "UnitFrameConfigChanged", true)
	Editor:CreateCheckBox(raid, true, 300, L["hideBlizzRaidFrames"], db.raid, "hideBlizzRaidFrames", "UnitFramesHideBlizzRaidFrames", "UnitFrameConfigChanged", true)

	local sortMethods = {
		["class"] = "Class",
		["role"]  = "Role",
		["name"]  = "Name",
		["group"] = "Group"
	}
	Editor:CreateDropDown(raid, false, 300, L["RaidSortMethod"], sortMethods, db.raid, "sortMethod", "UnitFramesSortMethod", "UnitFrameConfigChanged")

	Editor:CreateHorizLine(holder)
	Editor:CreateHeaderlabel(holder, L["Friendly"])
	local friendly = Editor:CreateGroup(holder, true, nil, {10, 0, 10, 10}, {0, 0, 0, 0})
	Editor:CreateCheckBox(friendly, false, 300, L["ShowFriendlyNPC"], db.friendly, "showFriendlyNPC", "UnitFramesShowFriendlyNPC", "UnitFrameConfigChanged", true)

	Editor:CreateHorizLine(holder)
	Editor:CreateHeaderlabel(holder, L["Container"])
	local container = Editor:CreateGroup(holder, true, nil, {10, 0, 10, 10}, {0, 0, 0, 0})
	local sb = Editor:CreateCheckBox(container, false, 300, L["showContainerBackgroundCB"], db.container, "showBackground", "UnitFramesShowBackground", "UnitFrameConfigChanged")
	Editor:CreateSlider(container, false, 300, L["Scale"], 0.5, 1.5, 0.05, true, db.container, "scale", "UnitFramesScale", "UnitFrameConfigChanged", true)
	Editor:CreateSlider(container, false, 300, L["MarginSize"], 0, 25, 1, false, db.container, "marginSize", "UnitFramesMarginSize", "UnitFrameConfigChanged")

	Editor:CreateHorizLine(holder)
	Editor:CreateHeaderlabel(holder, L["ContainerBackground"])

	local decor = Editor:CreateFrameDecorationEditor(holder, db.container, "ContBg", "UnitFrameConfigChanged")
	Editor:UpdateFrameDecoration(decor)

	sb:SetCallback("OnValueChanged", function(self, event, value)
		db.container.showBackground = value
		Healcsi:BroadcastEvent("UnitFramesShowBackground")
		Editor:UpdateFrameDecoration(decor)
	end)

	Editor:CreateHorizLine(holder)
	Editor:CreateHeaderlabel(holder, "Editor background")

	local background = Gui:Create("Healcsi-Background")
	background:SetWidth(350)
	background:SetHeight(54)
	background:SetList(Healcsi.Media.bgTextures)
	background:SetLabel("Background")
	background:SetValue(Healcsi.db.global.configWindow.bgFile)
	background:SetCallback("OnValueChanged", function(self, event, value)
		Healcsi.db.global.configWindow.bgFile = value
		self:SetValue(Healcsi.db.global.configWindow.bgFile)
		Healcsi:BroadcastEvent("EditorConfigChanged")
	end)
	holder:AddChild(background)

	
end

function UnitFrames:CreateEditor()

	local scrollFrame = Gui:Create("Healcsi-ScrollFrame")
	scrollFrame:SetLayout("Flow")
	scrollFrame:SetFullWidth(true)
	scrollFrame:SetFullHeight(true)
	self.scrollFrame = scrollFrame

	local parentFrame = Gui:Create("SimpleGroup")
	parentFrame:SetLayout("Flow")
	parentFrame:SetFullWidth(true)
	parentFrame:SetFullHeight(true)
	self.parentFrame = parentFrame

	self:CreateUnitFramesEditorView(parentFrame)

	scrollFrame:AddChild(parentFrame)
	return scrollFrame
end