local Healcsi     = Healcsi
local UnitFrames  = Healcsi.Modules.UnitFrames
local UnitButtons = Healcsi.Modules.UnitButtons
local Editor      = Healcsi.Editor

local RegisterStateDriver = RegisterStateDriver
local RegisterUnitWatch   = RegisterUnitWatch
local tinsert             = table.insert
local strlower            = string.lower

function UnitFrames:Party_UpdateLayout()
	if (not(self.Party)) then self:Party_CreateFrame() end

	local horiz, spacing = self:GetLayoutData()

	local i
	for i = 1, 5 do
		local playerButton = self.Party.playerButtons[i]
		local petButton    = self.Party.petButtons[i]

		if (i > 1) then playerButton:ClearAllPoints() end
		petButton:ClearAllPoints()

		if (horiz) then
			if (i > 1) then
				playerButton:SetPoint("LEFT", self.Party.playerButtons[i - 1], "RIGHT", spacing, 0)
			end
			petButton:SetPoint("TOP", playerButton, "BOTTOM", 0, -spacing)
		else
			if (i > 1) then
				playerButton:SetPoint("TOP", self.Party.playerButtons[i - 1], "BOTTOM", 0, -spacing)
			end
			petButton:SetPoint("LEFT", playerButton, "RIGHT", spacing, 0)
		end
	end
end

function UnitFrames:Party_GetFramesMatrix()
	if (not(self.Party)) then self:Party_CreateFrame() end

	local count = 0
	local hasPet

	local i
	for i = 1, 5 do
		if (self.Party.playerButtons[i]:IsVisible()) then
			count = i
			if (not(hasPet) and self.Party.petButtons[i]:IsVisible()) then
				hasPet = true
			end
		else
			break
		end
	end

	if (self:GetAlignHoriz()) then
		return count, hasPet and 2 or 1, hasPet
	else
		return hasPet and 2 or 1, count, hasPet
	end
end

function UnitFrames:Party_CreateFrame()
	local frame = self:Container_CreateGroupParent("HealcsiPartyFrame")
	RegisterStateDriver(frame, "visibility", "[group:raid] hide; [group:party] show; hide")

	local petFrame = CreateFrame("Frame", frame:GetName().."PetFrame", frame, "SecureFrameTemplate")

	self.Party = {}
	self.Party.frame = frame
	self.Party.petFrame = petFrame
	self.Party.playerButtons = {}
	self.Party.petButtons = {}

	local i
	for i = 0, 4 do

		local playerName = (i == 0 and "Player" or "Party"..i)
		local petName = (i == 0 and "Pet" or "PartyPet"..i)
		local playerUnit = strlower(playerName)
		local petUnit = strlower(petName)

		local playerButton = CreateFrame("Button", frame:GetName()..playerName, frame, "HealcsiUnitButtonTemplate")
		playerButton:SetAttribute("unit", playerUnit)

		local petButton = CreateFrame("Button", frame:GetName()..petName, petFrame, "HealcsiUnitButtonTemplate")
		petButton:SetAttribute("unit", petUnit)

		tinsert(self.Party.playerButtons, playerButton)
		tinsert(self.Party.petButtons, petButton)

		if i == 0 then
			playerButton:SetPoint("TOPLEFT")
			playerButton:Show()
			RegisterStateDriver(petButton, "visibility", "[nopet] hide; [vehicleui] hide; show")
		else
			RegisterUnitWatch(playerButton)
			RegisterStateDriver(petButton, "visibility", "[@"..petUnit..",noexists] hide; [@"..playerUnit..",unithasvehicleui] hide; show")
		end
	end

	self:Party_UpdateLayout()
end


Healcsi:RegisterEventCallback("UnitButtonConfigChanged", function()
	if (not(Editor.InCombat)) then
		UnitFrames:Party_UpdateLayout()
	end
end)

Healcsi:RegisterEventCallback("UnitFrameConfigChanged", function()
	if (not(Editor.InCombat)) then
		UnitFrames:Party_UpdateLayout()
		if (Healcsi.db.profile.UnitFrames.party.showPartyPets) then
			UnitFrames.Party.petFrame:Show()
		else
			UnitFrames.Party.petFrame:Hide()
		end
	end
end)

Healcsi:RegisterEventCallback("GlobalConfigRefresh", function()
	if (not(Editor.InCombat)) then
		UnitFrames:Party_UpdateLayout()
		if (Healcsi.db.profile.UnitFrames.party.showPartyPets) then
			UnitFrames.Party.petFrame:Show()
		else
			UnitFrames.Party.petFrame:Hide()
		end 
	end
end)