
local Healcsi     = Healcsi
local Console     = Healcsi.Console
local Media       = Healcsi.Media
local Gui         = Healcsi.Gui
local L           = Healcsi.Locale
local Modules     = Healcsi.Modules
local UnitFrames  = Healcsi.Modules.UnitFrames
local UnitButtons = Healcsi.Modules.UnitButtons
local Editor      = Healcsi.Editor


local GetNumGroupMembers, UnitIsGroupLeader, UnitIsRaidOfficer = GetNumGroupMembers, UnitIsGroupLeader, UnitIsRaidOfficer

local RegisterStateDriver   = RegisterStateDriver
local UnregisterStateDriver = UnregisterStateDriver
local InCombatLockdown      = InCombatLockdown

local function DisableBlizzardFrame(frame)
	if not frame then
		return
	end

	frame:UnregisterAllEvents()
	frame:SetScript("OnEvent", nil)
	frame:SetScript("OnUpdate", nil)
	frame:SetScript("OnSizeChanged", nil)
	frame:EnableMouse(false)
	frame:EnableKeyboard(false)
	frame:Hide()
	frame:SetAlpha(0)
	frame:SetScale(0.01)
	RegisterStateDriver(frame, "visibility", "hide")
end

function UnitFrames:Initialize()

	self:Container_CreateFrame()

	self:Solo_CreateFrame()
	self:Party_CreateFrame()
	self:Raid_CreateFrame()
	self:FriendlyNpc_CreateFrame()

	local blizzPartyParent = CreateFrame("Frame", "HealcsidDisableBlizzardPartyParentFrame", Healcsi.UIParent, "SecureFrameTemplate")
	blizzPartyParent:Hide()
	blizzPartyParent:RegisterEvent("VARIABLES_LOADED")
	self.blizzPartyParent = blizzPartyParent

	blizzPartyParent:SetScript("OnEvent", function(self)
		SetCVar("useCompactPartyFrames", "0")
		--SetCVar("raidFramesDisplayIncomingHeals", 1) -- To fix a Blizzard error introduced in patch 5.4 (cvar removed in patch 5.4.1)
	end)

	Healcsi:RegisterEventCallback("UnitFramesHideBlizzPartyFrames", function()
		local hide = Healcsi.db.profile.UnitFrames.party.hideBlizzPartyFrames
		local i

		for i = 1, 4 do
			local frame = _G["PartyMemberFrame"..i]
			if hide then
				frame:SetParent(blizzPartyParent)
			else
				frame:SetParent(Healcsi.UIParent)
			end
		end

		local bkgnd = PartyMemberBackground
		if bkgnd then
			if hide then
				bkgnd:SetParent(blizzPartyParent)
			else
				bkgnd:SetParent(Healcsi.UIParent)
			end
		end

	end)

	local hide = Healcsi.db.profile.UnitFrames.party.hideBlizzPartyFrames
	local i

	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		if hide then
			frame:SetParent(blizzPartyParent)
		else
			frame:SetParent(Healcsi.UIParent)
		end
	end

	local bkgnd = PartyMemberBackground
	if bkgnd then
		if hide then
			bkgnd:SetParent(blizzPartyParent)
		else
			bkgnd:SetParent(Healcsi.UIParent)
		end
	end

	hide = Healcsi.db.profile.UnitFrames.raid.hideBlizzRaidFrames

	if (hide) then
		DisableBlizzardFrame(CompactRaidFrameManager)
		DisableBlizzardFrame(CompactRaidFrameContainer)
		DisableBlizzardFrame(CompactUnitFrameProfilesGeneralOptionsFrame)
		DisableBlizzardFrame(CompactUnitFrameProfilesRaidStylePartyFrames)
		DisableBlizzardFrame(CompactUnitFrameProfilesProfileSelector)
		DisableBlizzardFrame(CompactUnitFrameProfilesSaveButton)
		DisableBlizzardFrame(CompactUnitFrameProfilesDeleteButton)
	end
	

	self.initDone = true
end

function UnitFrames:IsInitDone()
	return self.initDone
end

function UnitFrames:GetGroupStats()
	local num = GetNumGroupMembers()
	local group, leadship
	if IsInRaid() then
		group = "raid"
		if UnitIsGroupLeader("player") then
			leadship = "leader"
		elseif UnitIsRaidOfficer("player") then
			leadship = "officer"
		end
	elseif num > 0 then
		group = "party"
		if UnitIsGroupLeader("player") then
			leadship = "leader"
		end
	end

	return group, leadship, num
end

local function ContainerHolder_DoOnUpdate(self)
	local group = self:GetGroupStats()
	local method

	local cols, rows
	if group == "raid" then
		method = "Raid_GetFramesMatrix"
	elseif group == "party" then
		method = "Party_GetFramesMatrix"
	else
		method = "Solo_GetFramesMatrix"
	end

	local func = UnitFrames[method]
	if (type(func) == "function") then
		cols, rows = func(UnitFrames)
	else
		cols, rows = 1, 1
	end

	if (cols < 1) then cols = 1 end

	if (rows < 1) then rows = 1 end

	local unitSpacing = UnitButtons:GetUnitSpacing()
	local unitWidth   = UnitButtons:GetUnitWidth()
	local unitHeight  = UnitButtons:GetUnitHeight()

	local width = cols * (unitWidth + unitSpacing) - unitSpacing
	local height = rows * (unitHeight + unitSpacing) - unitSpacing

	self.frame.holder:SetSize(width, height)
	self.frame:SetClampRectInsets(0, width - 48, 0, 16 - height) -- Is this function protected? Currently not, but it should be...

end

local function ContainerHolder_OnUpdate(holder, elapsed)
	local self = holder.obj
	self.updateTimer = (self.updateTimer or 0) + elapsed
	if self.updateTimer > 0.2 then
		self.updateTimer = 0

		if (not(holder.needCalcSize)) then return end
		holder.needCalcSize = nil

		ContainerHolder_DoOnUpdate(holder.obj)
	end
end

local function Container_OnGroupRosterEvent(self, event)
	self.needRefresh = true
end

local function Container_GroupRosterUpdateWorkaround(self, elapsed)
	self.updateTimer = (self.updateTimer or 0) + elapsed
	if self.updateTimer > 0.2 then
		self.updateTimer = 0
		if (self.needRefresh and not(InCombatLockdown())) then
			if (UnitFrames.frame) then
				self.needRefresh = nil
				UnitFrames.frame:Hide()
				UnitFrames.frame:Show()
			end
		end
	end
end

local function Container_OnDragStart(self)
	if (not(Healcsi.db.profile.UnitFrames.general.lockPosition)) then
		if (not(UnitFrames.frame)) then UnitFrames:Container_CreateFrame() end
		UnitFrames.frame:StartMoving()
	end
end

local function Container_OnDragStop(self)
	if (not(UnitFrames.frame)) then UnitFrames:Container_CreateFrame() end
	UnitFrames.frame:StopMovingOrSizing()

	local frame = self.frame
	--local width, height = GetScreenWidth(), GetScreenHeight()
	--local top, left = frame:GetTop(), frame:GetRight(), frame:GetBottom(), frame:GetLeft()
	--top = height - top

	--self.Database.offset.left = math.floor(self.Database.offset.left + 0.5)
	--self.Database.offset.top = math.floor(self.Database.offset.top + 0.5)

	--frame:ClearAllPoints() -- Reset anchor, beacuse why not? Right?!
	--frame:SetPoint("TOPLEFT", TopParent, "BOTTOMLEFT", self.Database.offset.left, self.Database.offset.top)

end

function UnitFrames:RegisterMainFrameMover(frame)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", Container_OnDragStart)
	frame:SetScript("OnDragStop", Container_OnDragStop)
end

function UnitFrames:Container_CreateFrame()
	if self.frame then return end -- don't create it again!...

	local frame = CreateFrame("Frame", "HealcsiUnitFrameContainer", Healcsi.UIParent, "SecureFrameTemplate")
	frame.obj = self
	frame:SetSize(80, 50)
	frame:SetFrameStrata("LOW")
	frame:SetMovable(true)
	--frame:SetUserPlaced(true) -- use settings from config!
	frame:SetClampedToScreen(true) -- attempting to move or resize the frame beyond the edges of the screen will move/resize it no further than the edge of the screen closest to the mouse position

	local db = Healcsi.db.profile.UnitFrames.container

	if (db.offset.top ~= 0) or (db.offset.left ~= 0) then
		frame:SetPoint("TOPLEFT", Healcsi.UIParent, "BOTTOMLEFT", db.offset.left, db.offset.top)
	else
		frame:SetPoint("CENTER", Healcsi.UIParent, "CENTER", 0, 0)
	end

	local holder = CreateFrame("Frame", frame:GetName().."Holder", frame)
	holder:SetPoint("TOPLEFT")
	holder:SetFrameStrata("BACKGROUND")
	holder.obj = self
	holder:SetScript("OnUpdate", ContainerHolder_OnUpdate)
	holder.needCalcSize = true

	local decoration = CreateFrame("Frame", holder:GetName().."Decoration", holder)
	decoration:SetPoint("CENTER")
	decoration:SetFrameStrata("BACKGROUND")
	self:RegisterMainFrameMover(decoration)
	decoration.obj = self

	frame.holder = holder
	frame.decoration = decoration
	self.frame = frame

	-- Some bugs really need to be fixed by Blizzard:
	local rosterBugWorkaroundFrame = CreateFrame("Frame", nil, frame)
	rosterBugWorkaroundFrame:RegisterEvent("GROUP_ROSTER_UPDATE")
	rosterBugWorkaroundFrame:SetScript("OnEvent", Container_OnGroupRosterEvent)
	rosterBugWorkaroundFrame:SetScript("OnUpdate", Container_GroupRosterUpdateWorkaround)
	self.rosterBugWorkaroundFrame = rosterBugWorkaroundFrame

	self:UpdateContainer()
	self:UpdateContainerSize()
end

function UnitFrames:GetContainer()
	if (not(self.frame)) then self:Container_CreateFrame() end
	return self.frame
end

function UnitFrames:UpdateContainer()
	if (not(self.frame)) then self:Container_CreateFrame() end

	if (Editor.InCombat) then return end

	local db = Healcsi.db.profile.UnitFrames.container

	local edgeFile = nil
	local edgeSize = 0
	local bgFile = nil
	local tile = false

	if (db.showBackground) then

		-- try to use proper config only when it's enabled

		if (db.border.file ~= "None") then
			if (db.border.edgeSize ~= 0) then
				edgeFile = Healcsi:GetMedia("border", db.border.file)
				edgeSize = db.border.edgeSize
			end
		end

		if (db.background.file ~= "None") then
			bgFile = Healcsi:GetMedia("background", db.background.file)
		end

		if (db.background.tile == "tile") then
			tile = true
		end

	end

	local backdrop = {
		bgFile   = bgFile,
		tile     = tile,
		tileSize = db.background.tileSize,
		insets   = db.background.insets,
		edgeFile = edgeFile,
		edgeSize = edgeSize
	}

	self.frame.decoration:SetBackdrop(backdrop)
	self.frame.decoration:SetBackdropColor(Healcsi:ConvColorObj(db.background.color))
	self.frame.decoration:SetBackdropBorderColor(Healcsi:ConvColorObj(db.border.color))

	-- Scale
	self.frame:SetScale(db.scale)

	-- Mode
	self:UpdateShowMode()

	-- MarginSize

	local marginSize = db.marginSize
	self.frame.decoration:ClearAllPoints()
	self.frame.decoration:SetPoint("TOPLEFT", -marginSize, marginSize)
	self.frame.decoration:SetPoint("BOTTOMRIGHT", marginSize + 1, -marginSize - 1)

end

function UnitFrames:UpdateContainerSize()
	if (not(self.frame)) then self:Container_CreateFrame() end

	local holder = self.frame.holder
	holder.needCalcSize = true
end

function UnitFrames:IsShowSolo()
	return Healcsi.db.profile.UnitFrames.solo.showWhenNotGrouped
end

function UnitFrames:IsShowParty()
	return Healcsi.db.profile.UnitFrames.party.showWhenInParty
end

function UnitFrames:IsShowRaid()
	return Healcsi.db.profile.UnitFrames.raid.showWhenInRaid
end

function UnitFrames:UpdateShowMode()
	if (not(self.frame)) then self:Container_CreateFrame() end

	if (InCombatLockdown()) then return end

	local showSolo = self:IsShowSolo()
	local showParty = self:IsShowParty()
	local showRaid = self:IsShowRaid()

	if ( showSolo and showParty and showRaid ) then
		UnregisterStateDriver(self.frame, "visibility")
		self.frame:Show()
	elseif ( showSolo and showParty and not(showRaid) ) then
		RegisterStateDriver(self.frame, "visibility", "[nogroup] show; [group:party] show; hide")
	elseif (showSolo and not(showParty) and showRaid) then
		RegisterStateDriver(self.frame, "visibility", "[nogroup] show; [group:raid] show; hide")
	elseif ( showSolo and not(showParty) and not(showRaid) ) then
		RegisterStateDriver(self.frame, "visibility", "[nogroup] show; hide")
	elseif ( not(showSolo) and showParty and showRaid ) then
		RegisterStateDriver(self.frame, "visibility", "[group] show; hide")
	elseif ( not(showSolo) and showParty and not(showRaid) ) then
		RegisterStateDriver(self.frame, "visibility", "[group:party] show; hide")
	elseif ( not(showSolo) and not(showParty) and showRaid ) then
		RegisterStateDriver(self.frame, "visibility", "[group:raid] show; hide")
	else
		RegisterStateDriver(self.frame, "visibility", "hide")
	end
end

function UnitFrames:GetAlignHoriz()
	return Healcsi.db.profile.UnitFrames.general.alignHorizontally
end

function UnitFrames:GetKeepGroupsTogether()
	return Healcsi.db.profile.UnitFrames.raid.keepGroupsTogether
end

function UnitFrames:GetLayoutData()
	return self:GetAlignHoriz(), UnitButtons:GetUnitSpacing()
end

function UnitFrames:Container_CreateGroupParent(name, templates)
	if (not(self.frame)) then self:Container_CreateFrame() end

	local parent = CreateFrame("Frame", name, self.frame, templates or "SecureFrameTemplate")
	parent:SetSize(80, 50)
	parent:SetPoint("TOPLEFT")
	return parent

end

Healcsi:RegisterEventCallback("UpdateContainerSize", function()
	UnitFrames:UpdateContainerSize()
end)

Healcsi:RegisterEventCallback("GlobalConfigRefresh", function()
	UnitFrames:UpdateShowMode()
	UnitFrames:UpdateContainer()
	UnitFrames:UpdateContainerSize()
end)

Healcsi:RegisterEventCallback("UnitButtonConfigChanged", function()
	UnitFrames:UpdateContainer()
	UnitFrames:UpdateContainerSize()
end)

Healcsi:RegisterEventCallback("UnitFrameConfigChanged", function()
	UnitFrames:UpdateShowMode()
	UnitFrames:UpdateContainer()
	UnitFrames:UpdateContainerSize()
end)